// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns / 100ps

module  bts_config (
        //Reset&Clock
        input  wire        cpu_1v2_resetn          , // Reset, low active
        input  wire        clk_sys_100m_p          , // System clock, 100MHz

        //I2C interfaces
        output wire        fpga_i2c_en             , // FPGA I2C interface #1 enable, DIPSW ctrl
        inout  wire        fpga_i2c1_scl           , // FPGA I2C interface #1 SCL
        inout  wire        fpga_i2c1_sda           , // FPGA I2C interface #1 SDA
        output wire        fpga_i2c2_en            , // FPGA I2C interface #2 enable, DIPSW ctrl
        inout  wire        fpga_i2c2_scl           , // FPGA I2C interface #2 SCL
        inout  wire        fpga_i2c2_sda           , // FPGA I2C interface #2 SDA
        inout  wire        fpga_dimm_scl           , // FPGA I2C interface dedicated for RDIMM access
        inout  wire        fpga_dimm_sda           , // FPGA I2C interface dedicated for RDIMM access

        //QSPI/SPI master
        output             m10_1v2_clk             , // SPI_CLK, output
        output             m10_1v2_ssn             , // SPI_NSS, input
        input              m10_1v2_data0           , // SPI_MISO, output
        output             m10_1v2_data1           , // SPI_MOSI, output
        inout              m10_1v2_data2           , // Reserved
        inout              m10_1v2_data3           , // Reserved
        input              m10_1v2_gpio0           , // NIOS heart beat
        input              m10_1v2_gpio1           , // Reserved for interrupt from MAX10 to FPGA

        //GPIOs
        output wire [3:0]  fpga_led                , // FPGA user LEDs

        //QSFPDD LEDs & control signals
        output wire [2:0]  qsfpdd0_fpga_led        , // QSFPDD0 LEDs
        output wire [2:0]  qsfpdd1_fpga_led        , // QSFPDD1 LEDs
        output  wire       qsfpdd_12v_port_en      , // Enabling of I2C controller U14, external pull-up
        input   wire       qsfpdd_1v2_port_int_n     // Interrupt from I2C controller U14
        );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Signal definitions
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
wire          ninit_done/* synthesis keep */;
wire          global_resetn/* synthesis keep */;
wire          fpga_i2c1_scl_in/* synthesis keep */;
wire          fpga_i2c1_sda_in/* synthesis keep */;
wire          fpga_i2c1_scl_oe/* synthesis keep */;
wire          fpga_i2c1_sda_oe/* synthesis keep */;
wire          fpga_i2c2_scl_in/* synthesis keep */;
wire          fpga_i2c2_sda_in/* synthesis keep */;
wire          fpga_i2c2_scl_oe/* synthesis keep */;
wire          fpga_i2c2_sda_oe/* synthesis keep */;
wire          fpga_dimm_scl_in/* synthesis keep */;
wire          fpga_dimm_sda_in/* synthesis keep */;
wire          fpga_dimm_scl_oe/* synthesis keep */;
wire          fpga_dimm_sda_oe/* synthesis keep */;

wire [31:0]   av_readdata_0/* synthesis keep */;
wire [31:0]   av_readdata_1/* synthesis keep */;
wire [31:0]   av_readdata_2/* synthesis keep */;
wire [31:0]   av_readdata_3/* synthesis keep */;
wire [31:0]   av_writedata_0/* synthesis keep */; // FPGA LED control
wire [31:0]   av_writedata_1/* synthesis keep */; // FPGA QSFPDD LED control
//wire [31:0] av_writedata_2/* synthesis keep */; // FPGA QSFPDD controller and module management
//wire [31:0] av_writedata_3/* synthesis keep */;
//wire [31:0] av_writedata_4/* synthesis keep */;
//wire [31:0] av_writedata_5/* synthesis keep */;
//wire [31:0] av_writedata_6/* synthesis keep */;
//wire [31:0] av_writedata_7/* synthesis keep */;
wire [1:0]    receiver_irq/* synthesis keep */;
wire          outclk0_100m/*100m*/;
wire          outclk1_50m/*50m*/;

reg     [26:0] heart_beat_cnt/* synthesis keep */;
wire          iopll_lock       /* synthesis keep */;
wire          global_resetn1   /* synthesis keep */;

//----------------------------------------------------------------------------------------------------------
// Intel Agilex Reset Release
//----------------------------------------------------------------------------------------------------------
reset_release reset_release_inst 
              (
              .ninit_done (ninit_done)  // ninit_done.ninit_done, low active
              );

assign global_resetn = !ninit_done & cpu_1v2_resetn;
assign global_resetn1 = global_resetn & iopll_lock;

//assign global_resetn = cpu_1v2_resetn;

//----------------------------------------------------------------------------------------------------------
// Glue logic for heart beat
//----------------------------------------------------------------------------------------------------------
always @(posedge outclk1_50m or negedge global_resetn1)
        if (!global_resetn1)
            heart_beat_cnt <= 27'h0; //0x7FFFFFF
        else
            heart_beat_cnt <= heart_beat_cnt + 1'b1;

// iopll----------------------------------------------------------------------------------------------------
iopll iopll_0 (
		.rst      (!global_resetn),      //   input,  width = 1,   reset.reset
		.refclk   (clk_sys_100m_p),   //   input,  width = 1,  refclk.clk
		.locked   (iopll_lock),   //  output,  width = 1,  locked.export
		.outclk_0 (outclk0_100m), //  output,  width = 1, outclk0.clk 100m
		.outclk_1 (outclk1_50m)  //  output,  width = 1, outclk1.clk  50m
	);
// ---------------------------------------------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Qsys module
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
q_sys   q_sys_inst0 (
        .clk_50_clk                       (outclk1_50m     ), //           clk_50.clk
        .reset_50_reset_n                 (global_resetn1         ), //         reset_50.reset_n
        .i2c_0_i2c_serial_sda_in          (fpga_i2c1_sda_in      ), // i2c_0_i2c_serial.sda_in
        .i2c_0_i2c_serial_scl_in          (fpga_i2c1_scl_in      ), //                 .scl_in
        .i2c_0_i2c_serial_sda_oe          (fpga_i2c1_sda_oe      ), //                 .sda_oe
        .i2c_0_i2c_serial_scl_oe          (fpga_i2c1_scl_oe      ), //                 .scl_oe
        .i2c_1_i2c_serial_sda_in          (fpga_i2c2_sda_in      ), // i2c_1_i2c_serial.sda_in
        .i2c_1_i2c_serial_scl_in          (fpga_i2c2_scl_in      ), //                 .scl_in
        .i2c_1_i2c_serial_sda_oe          (fpga_i2c2_sda_oe      ), //                 .sda_oe
        .i2c_1_i2c_serial_scl_oe          (fpga_i2c2_scl_oe      ), //                 .scl_oe
        .i2c_2_i2c_serial_sda_in          (fpga_dimm_sda_in      ), // i2c_2_i2c_serial.sda_in
        .i2c_2_i2c_serial_scl_in          (fpga_dimm_scl_in      ), //                 .scl_in
        .i2c_2_i2c_serial_sda_oe          (fpga_dimm_sda_oe      ), //                 .sda_oe
        .i2c_2_i2c_serial_scl_oe          (fpga_dimm_scl_oe      ), //                 .scl_oe
        .spi_0_external_MISO              (m10_1v2_data0         ), //   spi_0_external.MISO
        .spi_0_external_MOSI              (m10_1v2_data1         ), //                 .MOSI
        .spi_0_external_SCLK              (m10_1v2_clk           ), //                 .SCLK
        .spi_0_external_SS_n              (m10_1v2_ssn           ), //                 .SS_n
        .av_readdata_0_export             (av_readdata_0         ), //    av_readdata_0.export
        .av_readdata_1_export             (av_readdata_1         ), //    av_readdata_1.export
        .av_readdata_2_export             (av_readdata_2         ), //    av_readdata_2.export
        .av_readdata_3_export             (av_readdata_3         ), //    av_readdata_3.export
        .av_writedata_0_export            (av_writedata_0        ), //   av_writedata_0.export
        .av_writedata_1_export            (av_writedata_1        ), //   av_writedata_1.export
      //.av_writedata_2_export            (av_writedata_2        ), //   av_writedata_2.export
      //.av_writedata_3_export            (av_writedata_3        ), //   av_writedata_3.export
      //.av_writedata_4_export            (av_writedata_4        ), //   av_writedata_4.export
      //.av_writedata_5_export            (av_writedata_5        ), //   av_writedata_5.export
      //.av_writedata_6_export            (av_writedata_6        ), //   av_writedata_6.export
      //.av_writedata_7_export            (av_writedata_7        ), //   av_writedata_7.export
        .receiver_irq_irq                 (receiver_irq          )  //     receiver_irq.irq
	);

assign av_readdata_0   = {{30{1'b0}}, m10_1v2_gpio1, m10_1v2_gpio0};
assign av_readdata_1   = 32'h00000000;
assign av_readdata_2   = 32'h00000000;
assign av_readdata_3   = 32'h00000000;

assign receiver_irq    = {!qsfpdd_1v2_port_int_n, 1'b0};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output assignments control
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
assign fpga_i2c1_scl_in     = fpga_i2c1_scl;
assign fpga_i2c1_sda_in     = fpga_i2c1_sda;
assign fpga_i2c1_scl        = fpga_i2c1_scl_oe ? 1'b0 : 1'bz;
assign fpga_i2c1_sda        = fpga_i2c1_sda_oe ? 1'b0 : 1'bz;

assign fpga_i2c2_scl_in     = fpga_i2c2_scl;
assign fpga_i2c2_sda_in     = fpga_i2c2_sda;
assign fpga_i2c2_scl        = fpga_i2c2_scl_oe ? 1'b0 : 1'bz;
assign fpga_i2c2_sda        = fpga_i2c2_sda_oe ? 1'b0 : 1'bz;

assign fpga_dimm_scl_in     = fpga_dimm_scl;
assign fpga_dimm_sda_in     = fpga_dimm_sda;
assign fpga_dimm_scl        = fpga_dimm_scl_oe ? 1'b0 : 1'bz;
assign fpga_dimm_sda        = fpga_dimm_sda_oe ? 1'b0 : 1'bz;

assign fpga_i2c_en          = 1'bz;                            // DIPSW control externally
assign fpga_i2c2_en         = 1'bz;                            // DIPSW control externally

assign fpga_led[0]          = av_writedata_0[0] ? 1'b1 : 1'b0; // av_writedata_0[0]: 1'b0 by default
assign fpga_led[1]          = av_writedata_0[1] ? 1'b1 : 1'b0; // av_writedata_0[0]: 1'b0 by default
assign fpga_led[2]          = av_writedata_0[2] ? 1'b1 : 1'b0; // av_writedata_0[0]: 1'b0 by default
//assign fpga_led[3]          = av_writedata_0[3] ? 1'b1 : 1'b0; // av_writedata_0[0]: 1'b0 by default
assign fpga_led[3]          = heart_beat_cnt[26]; // av_writedata_0[0]: 1'b0 by default

assign qsfpdd0_fpga_led[0]  = av_writedata_1[0] ? 1'b1 : 1'b0; // av_writedata_1[0]: 1'b0 by default
assign qsfpdd0_fpga_led[1]  = av_writedata_1[1] ? 1'b1 : 1'b0; // av_writedata_1[1]: 1'b0 by default
assign qsfpdd0_fpga_led[2]  = av_writedata_1[2] ? 1'b1 : 1'b0; // av_writedata_1[2]: 1'b0 by default
assign qsfpdd1_fpga_led[0]  = av_writedata_1[3] ? 1'b1 : 1'b0; // av_writedata_1[3]: 1'b0 by default
assign qsfpdd1_fpga_led[1]  = av_writedata_1[4] ? 1'b1 : 1'b0; // av_writedata_1[4]: 1'b0 by default
assign qsfpdd1_fpga_led[2]  = av_writedata_1[5] ? 1'b1 : 1'b0; // av_writedata_1[5]: 1'b0 by default

assign qsfpdd_12v_port_en   = 1'bz;                            // Enabling of I2C controller U14, external pull-up

endmodule


	
