-- (C) 2001-2024 Intel Corporation. All rights reserved.
-- Your use of Intel Corporation's design tools, logic functions and other 
-- software and tools, and its AMPP partner logic functions, and any output 
-- files from any of the foregoing (including device programming or simulation 
-- files), and any associated documentation or information are expressly subject 
-- to the terms and conditions of the Intel Program License Subscription 
-- Agreement, Intel FPGA IP License Agreement, or other applicable 
-- license agreement, including, without limitation, that your use is for the 
-- sole purpose of programming logic devices manufactured by Intel and sold by 
-- Intel or its authorized distributors.  Please refer to the applicable 
-- agreement for further details.



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_UNSIGNED.all;

						
entity product_info is
	port (
		clk: in std_logic;
		reset_n : in std_logic;
		
		chipselect_n : in std_logic; -- chip select signal
		read_n : in std_logic; -- read enable
		write_n : in std_logic; -- read enable
		av_address : in std_logic_vector(3 downto 0);
		av_data_read : out std_logic_vector(31 downto 0);
		av_data_write : in std_logic_vector(31 downto 0);
        av_readdata_0 : in std_logic_vector(31 downto 0);
        av_readdata_1 : in std_logic_vector(31 downto 0);
        av_readdata_2 : in std_logic_vector(31 downto 0);
        av_readdata_3 : in std_logic_vector(31 downto 0);
        av_writedata_0 : out std_logic_vector(31 downto 0);
        av_writedata_1 : out std_logic_vector(31 downto 0);
        av_writedata_2 : out std_logic_vector(31 downto 0);
        av_writedata_3 : out std_logic_vector(31 downto 0);
        av_writedata_4 : out std_logic_vector(31 downto 0);
        av_writedata_5 : out std_logic_vector(31 downto 0);
        av_writedata_6 : out std_logic_vector(31 downto 0);
        av_writedata_7 : out std_logic_vector(31 downto 0)
	);
end product_info;

architecture rtl of product_info is
--	constant sof : std_logic_vector(31 downto 0):=x"4750494F"; -- GPIO project
--	constant sof : std_logic_vector(31 downto 0):=x"56494445"; -- VIDE project
--	constant sof : std_logic_vector(31 downto 0):=x"48534D43"; -- HSMC project
--	constant sof : std_logic_vector(31 downto 0):=x"51445232"; -- QDR2 project
--	constant sof : std_logic_vector(31 downto 0):=x"44445234"; -- DDR4 project
--	constant sof : std_logic_vector(31 downto 0):=x"44445233"; -- DDR3 project
--	constant sof : std_logic_vector(31 downto 0):=x"44445232"; -- DDR2 project

constant sof     : std_logic_vector(31 downto 0):=x"4750494F"; -- GPIO project
constant board   : std_logic_vector(31 downto 0):=x"41474938"; -- AGI8: Agilex-I FM85 devkit
constant product : std_logic_vector(31 downto 0):=x"00005e81"; -- N24193-001 Agilex-I FPGA Dev Kit
constant version : std_logic_vector(31 downto 0):=x"00000001"; -- to indicate this is dev1

signal av_writedata_0_int : std_logic_vector(31 downto 0);
signal av_writedata_1_int : std_logic_vector(31 downto 0);
signal av_writedata_2_int : std_logic_vector(31 downto 0);
signal av_writedata_3_int : std_logic_vector(31 downto 0);
signal av_writedata_4_int : std_logic_vector(31 downto 0);
signal av_writedata_5_int : std_logic_vector(31 downto 0);
signal av_writedata_6_int : std_logic_vector(31 downto 0);
signal av_writedata_7_int : std_logic_vector(31 downto 0);

begin

	process(reset_n, clk)begin
		if(reset_n = '0')then
			av_data_read <= (others => '0');
		elsif(clk'event and clk = '1')then
			if(chipselect_n = '0' and read_n = '0')then
				case av_address is
					when "0000"  => av_data_read <= product;
					when "0001"  => av_data_read <= board;
					when "0010"  => av_data_read <= sof;
					when "0011"  => av_data_read <= version;
					when "0100"  => av_data_read <= av_readdata_0;
					when "0101"  => av_data_read <= av_readdata_1;
					when "0110"  => av_data_read <= av_readdata_2;
					when "0111"  => av_data_read <= av_readdata_3;
					when "1000"  => av_data_read <= av_writedata_0_int;
					when "1001"  => av_data_read <= av_writedata_1_int;
					when "1010"  => av_data_read <= av_writedata_2_int;
					when "1011"  => av_data_read <= av_writedata_3_int;
					when "1100"  => av_data_read <= av_writedata_4_int;
					when "1101"  => av_data_read <= av_writedata_5_int;
					when "1110"  => av_data_read <= av_writedata_6_int;
					when "1111"  => av_data_read <= av_writedata_7_int;
					when others => av_data_read <= (others => '0');
				end case;
			end if;
		end if;
	end process;

	process(reset_n, clk)begin
		if(reset_n = '0')then
			av_writedata_0_int <= (others => '0');                    -- FPGA LED control
			av_writedata_1_int <= (others => '0');                    -- FPGA QSFPDD LED control
			av_writedata_2_int <= "00000000000000010000000000000001"; -- QSFPDD low power mode control & controller reset
			av_writedata_3_int <= (others => '0');
			av_writedata_4_int <= (others => '0');
			av_writedata_5_int <= (others => '0');
			av_writedata_6_int <= (others => '0');
			av_writedata_7_int <= (others => '0');
		elsif(clk'event and clk = '1')then
			if(chipselect_n = '0' and write_n = '0')then
				case av_address is
					when "1000"  => av_writedata_0_int <= av_data_write;
					when "1001"  => av_writedata_1_int <= av_data_write;
					when "1010"  => av_writedata_2_int <= av_data_write;
					when "1011"  => av_writedata_3_int <= av_data_write;
					when "1100"  => av_writedata_4_int <= av_data_write;
					when "1101"  => av_writedata_5_int <= av_data_write;
					when "1110"  => av_writedata_6_int <= av_data_write;
					when "1111"  => av_writedata_7_int <= av_data_write;
					when others  => av_writedata_7_int <= (others => '0');
				end case;
			end if;
		end if;
	end process;

av_writedata_0 <= av_writedata_0_int;
av_writedata_1 <= av_writedata_1_int;
av_writedata_2 <= av_writedata_2_int;
av_writedata_3 <= av_writedata_3_int;
av_writedata_4 <= av_writedata_4_int;
av_writedata_5 <= av_writedata_5_int;
av_writedata_6 <= av_writedata_6_int;
av_writedata_7 <= av_writedata_7_int;

end rtl;

