// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



`timescale 1 ps / 1 ps
module  ddr4 (
        //Reset&Clock
        input  wire        cpu_1v2_resetn        , // Reset, low active
        input  wire        clk_sys_100m_p        , // System clock, 100MHz

        //I2C interfaces
        output wire        fpga_i2c_en           , // FPGA I2C interface #1 enable, DIPSW ctrl
        inout  wire        fpga_i2c1_scl         , // FPGA I2C interface #1 SCL
        inout  wire        fpga_i2c1_sda         , // FPGA I2C interface #1 SDA
        output wire        fpga_i2c2_en          , // FPGA I2C interface #2 enable, DIPSW ctrl
        inout  wire        fpga_i2c2_scl         , // FPGA I2C interface #2 SCL
        inout  wire        fpga_i2c2_sda         , // FPGA I2C interface #2 SDA
//        inout  wire        fpga_dimm_scl         , // FPGA I2C interface dedicated for RDIMM access
//        inout  wire        fpga_dimm_sda         , // FPGA I2C interface dedicated for RDIMM access

        //GPIOs
        output wire [3:0]  fpga_led              , // FPGA user LEDs

        //Memory interface: DDR4 x72 RDIMM
        input  wire        clk_ddr4_ch3_p        , // mem_ref_clk.clk
        output wire [0:0]  ddr4_dimm2_ck_p        , // mem.mem_ck
        output wire [0:0]  ddr4_dimm2_ck_n        , // .mem_ck_n
        output wire [16:0] ddr4_ddimm2_a          , // .mem_a
        output wire        ddr4_ddimm2_act_n      , // .mem_act_n
        output wire [1:0]  ddr4_ddimm2_ba         , // .mem_ba
        output wire [1:0]  ddr4_ddimm2_bg         , // .mem_bg
        output wire [0:0]  ddr4_dimm2_cke        , // .mem_cke     
        output wire [0:0]  ddr4_dimm2_cs_n       , // .mem_cs_n    
        output wire [0:0]  ddr4_dimm2_odt        , // .mem_odt     
        output wire        ddr4_ddimm2_reset_n    , // .mem_reset_n
        output wire        ddr4_ddimm2_par        , // .mem_par
        input  wire        ddr4_ddimm2_alert_n    , // .mem_alert_n
        inout  wire [8:0]  ddr4_ddimm2_dqs_p      , // .mem_dqs
        inout  wire [8:0]  ddr4_ddimm2_dqs_n      , // .mem_dqs_n
        inout  wire [8:0]  ddr4_ddimm2_dbi_n      , // .mem_dbi_n
        inout  wire [17:9] ddr4_ddimm2_tdqs_n     , // .mem_tdqs_n
        inout  wire [71:0] ddr4_ddimm2_dq         , // .mem_dq
        input  wire        ddr4_ddimm2_rzq          // .mem_rzq
        );

//----------------------------------------------------------------------------------------------------------
// Singal definitions
//----------------------------------------------------------------------------------------------------------
wire           ninit_done/* synthesis keep */;
wire           global_resetn/* synthesis keep */;
reg     [26:0] heart_beat_cnt/* synthesis keep */;
wire    [2:0]  test_status/* synthesis keep */;
wire  clk_sys_100m;
wire  clk_sys_50m;
wire          iopll_lock       /* synthesis keep */;
wire          global_resetn1   /* synthesis keep */;
//----------------------------------------------------------------------------------------------------------
// Intel Agilex Reset Release
//----------------------------------------------------------------------------------------------------------
reset_release reset_release_inst 
              (
              .ninit_done (ninit_done)  // ninit_done.ninit_done
              );

assign global_resetn = !ninit_done & cpu_1v2_resetn;
assign global_resetn1 = global_resetn & iopll_lock;

iopll iopll_u0 (
		.rst      (!global_resetn),       //   input,  width = 1, reset.reset
		.refclk   (clk_sys_100m_p),       //   input,  width = 1, refclk.clk
		.locked   (iopll_lock),                     //  output,  width = 1, locked.export
		.outclk_0 (clk_sys_100m) ,        //  output,  width = 1, outclk0.clk
		.outclk_1 (clk_sys_50m)           //  output,  width = 1, outclk1.clk
	);
	

//assign global_resetn = cpu_1v2_resetn;

//----------------------------------------------------------------------------------------------------------
// Glue logic for heart beat
//----------------------------------------------------------------------------------------------------------
always @(posedge clk_sys_50m or negedge global_resetn1)
        if (!global_resetn1)
            heart_beat_cnt <= 27'h0; //0x7FFFFFF
        else
            heart_beat_cnt <= heart_beat_cnt + 1'b1;

//----------------------------------------------------------------------------------------------------------
// DDR4 example design
//----------------------------------------------------------------------------------------------------------
ed_synth    ed_synth_u0 (
            .clk_50m_clk                              (clk_sys_50m    ), //                           clk_50m.clk
            .global_reset_reset_n                     (global_resetn1        ), //                      global_reset.reset_n
          //.clock_bridge_100mhz_clk                  (clk_sys_100m_p       ), //               clock_bridge_100mhz.clk
          //.reset_bridge_100mhz_reset_n              (global_resetn1        ), //               reset_bridge_100mhz.reset_n
            .clock_bridge_emif_refclk_clk             (clk_ddr4_ch3_p       ), //          clock_bridge_emif_refclk.clk
            .emif_fm_0_oct_oct_rzqin                  (ddr4_ddimm2_rzq       ), //                     emif_fm_0_oct.oct_rzqin
            .emif_fm_0_mem_mem_ck                     (ddr4_dimm2_ck_p     ), //                     emif_fm_0_mem.mem_ck
            .emif_fm_0_mem_mem_ck_n                   (ddr4_dimm2_ck_n     ), //                                  .mem_ck_n
            .emif_fm_0_mem_mem_a                      (ddr4_ddimm2_a         ), //                                  .mem_a
            .emif_fm_0_mem_mem_act_n                  (ddr4_ddimm2_act_n     ), //                                  .mem_act_n
            .emif_fm_0_mem_mem_ba                     (ddr4_ddimm2_ba        ), //                                  .mem_ba
            .emif_fm_0_mem_mem_bg                     (ddr4_ddimm2_bg        ), //                                  .mem_bg
            .emif_fm_0_mem_mem_cke                    (ddr4_dimm2_cke      ), //                                  .mem_cke
            .emif_fm_0_mem_mem_cs_n                   (ddr4_dimm2_cs_n     ), //                                  .mem_cs_n
            .emif_fm_0_mem_mem_odt                    (ddr4_dimm2_odt      ), //                                  .mem_odt
            .emif_fm_0_mem_mem_reset_n                (ddr4_ddimm2_reset_n   ), //                                  .mem_reset_n
            .emif_fm_0_mem_mem_par                    (ddr4_ddimm2_par       ), //                                  .mem_par
            .emif_fm_0_mem_mem_alert_n                (ddr4_ddimm2_alert_n   ), //                                  .mem_alert_n
            .emif_fm_0_mem_mem_dqs                    ({ddr4_ddimm2_dbi_n[8],  ddr4_ddimm2_dqs_p[8], 
                                                        ddr4_ddimm2_dbi_n[7],  ddr4_ddimm2_dqs_p[7],		
     												    ddr4_ddimm2_dbi_n[6],  ddr4_ddimm2_dqs_p[6],
													    ddr4_ddimm2_dbi_n[5],  ddr4_ddimm2_dqs_p[5],
													    ddr4_ddimm2_dbi_n[4],  ddr4_ddimm2_dqs_p[4],
													    ddr4_ddimm2_dbi_n[3],  ddr4_ddimm2_dqs_p[3],
													    ddr4_ddimm2_dbi_n[2],  ddr4_ddimm2_dqs_p[2],
													    ddr4_ddimm2_dbi_n[1],  ddr4_ddimm2_dqs_p[1],													   
													    ddr4_ddimm2_dbi_n[0],  ddr4_ddimm2_dqs_p[0]}),   //.mem_dqs                                                                 .mem_dqs
            .emif_fm_0_mem_mem_dqs_n                  ({ddr4_ddimm2_tdqs_n[17],  ddr4_ddimm2_dqs_n[8],
                                                        ddr4_ddimm2_tdqs_n[16], ddr4_ddimm2_dqs_n[7],	
                                                        ddr4_ddimm2_tdqs_n[15], ddr4_ddimm2_dqs_n[6],
                                                        ddr4_ddimm2_tdqs_n[14], ddr4_ddimm2_dqs_n[5],
                                                        ddr4_ddimm2_tdqs_n[13], ddr4_ddimm2_dqs_n[4],
                                                        ddr4_ddimm2_tdqs_n[12], ddr4_ddimm2_dqs_n[3],
                                                        ddr4_ddimm2_tdqs_n[11], ddr4_ddimm2_dqs_n[2],
                                                        ddr4_ddimm2_tdqs_n[10], ddr4_ddimm2_dqs_n[1],	
			                                            ddr4_ddimm2_tdqs_n[9], ddr4_ddimm2_dqs_n[0]}),  //.mem_dqs_n                                  
             .emif_fm_0_mem_mem_dq                     (ddr4_ddimm2_dq       ), //                     .mem_dq
 //         .emif_fm_0_mem_mem_dbi_n                  (ddr4_ddimm2_dbi_n     ), //                     .mem_dbi_n
            .local_reset_req                          (global_resetn1       ), //                     local_reset_req.local_reset_req
            .local_reset_done                         (test_status[2]       ), //                     local_reset_status.local_reset_done
            .master_driver_msgdma_0_error_mon_export  (test_status[1]       ), //                     master_driver_msgdma_0_error_mon.export
            .master_driver_msgdma_0_status_mon_export (test_status[0]       )  //                     master_driver_msgdma_0_status_mon.export
            );

//----------------------------------------------------------------------------------------------------------
// Output assignments control
//----------------------------------------------------------------------------------------------------------
assign fpga_led[3:0]  = {heart_beat_cnt[26], test_status[2], !test_status[1], test_status[0]};

assign fpga_i2c_en    = 1'bz;
assign fpga_i2c1_scl  = 1'bz;
assign fpga_i2c1_sda  = 1'bz;
assign fpga_i2c2_en   = 1'bz;
assign fpga_i2c2_scl  = 1'bz;
assign fpga_i2c2_sda  = 1'bz;
assign fpga_dimm_scl  = 1'bz;
assign fpga_dimm_sda  = 1'bz;

endmodule
