// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.

// FM85 golden top
`timescale 1ns / 100ps
module golden_top  ( 
		//Reset&Clock
		input wire         cpu_1v2_resetn          ,// Reset, low active 
		input wire         clk_sys_100m_p          ,// System clock, 100MHz

		//I2C interface
        output wire        fpga_i2c_en             , // FPGA I2C interface #1 enable, DIPSW ctrl
        inout  wire        fpga_i2c1_scl           , // FPGA I2C interface #1 SCL
        inout  wire        fpga_i2c1_sda           , // FPGA I2C interface #1 SDA
        output wire        fpga_i2c2_en            , // FPGA I2C interface #2 enable, DIPSW ctrl
        inout  wire        fpga_i2c2_scl           , // FPGA I2C interface #2 SCL
        inout  wire        fpga_i2c2_sda           , // FPGA I2C interface #2 SDA
        inout  wire        fpga_dimm_scl           , // FPGA I2C interface dedicated for RDIMM access
        inout  wire        fpga_dimm_sda           , // FPGA I2C interface dedicated for RDIMM access


        //GPIOs
        output wire [3:0]  fpga_led                , // FPGA user LEDs

		//Memory interface: DDR4 x72 RDIMM
        input  wire        clk_ddr4_ch2_p        , // mem_ref_clk.clk
        output wire [0:0]  ddr4_dimm_ck_p        , // mem.mem_ck
        output wire [0:0]  ddr4_dimm_ck_n        , // .mem_ck_n
        output wire [17:0] ddr4_ddimm_a          , // .mem_a
        output wire        ddr4_ddimm_act_n      , // .mem_act_n
        output wire [1:0]  ddr4_ddimm_ba         , // .mem_ba
        output wire [1:0]  ddr4_ddimm_bg         , // .mem_bg
        output wire [1:0]  ddr4_ddimm_c          , // .mem_c
        output wire [1:0]  ddr4_dimm_cke         , // .mem_cke     
        output wire [1:0]  ddr4_dimm_cs_n        , // .mem_cs_n    
        output wire [1:0]  ddr4_dimm_odt         , // .mem_odt     
        output wire        ddr4_ddimm_reset_n    , // .mem_reset_n
        output wire        ddr4_ddimm_par        , // .mem_par
        input  wire        ddr4_ddimm_alert_n    , // .mem_alert_n
        inout  wire [8:0]  ddr4_ddimm_dqs_p      , // .mem_dqs
        inout  wire [8:0]  ddr4_ddimm_dqs_n      , // .mem_dqs_n
        inout  wire [8:0]  ddr4_ddimm_dbi_n      , // .mem_dbi_n
        inout  wire [17:9] ddr4_ddimm_tdqs_n     , // .mem_tdqs_n
        inout  wire        fpga_ddimm_event_n    , // .mem_event_n
        inout  wire        fpga_ddimm_save_n     , // .mem_save_n
        inout  wire [71:0] ddr4_ddimm_dq         , // .mem_dq
        input  wire        ddr4_ddimm_rzq        , // .mem_rzq
		
        //Memory interface: DDR4 x72 RDIMM
        input  wire        clk_ddr4_ch3_p        , // mem_ref_clk.clk
        output wire [0:0]  ddr4_dimm2_ck_p        , // mem.mem_ck
        output wire [0:0]  ddr4_dimm2_ck_n        , // .mem_ck_n
        output wire [17:0] ddr4_ddimm2_a          , // .mem_a
        output wire        ddr4_ddimm2_act_n      , // .mem_act_n
        output wire [1:0]  ddr4_ddimm2_ba         , // .mem_ba
        output wire [1:0]  ddr4_ddimm2_bg         , // .mem_bg
        output wire [1:0]  ddr4_dimm2_cke        , // .mem_cke     
        output wire [1:0]  ddr4_dimm2_cs_n       , // .mem_cs_n    
        output wire [1:0]  ddr4_dimm2_odt        , // .mem_odt     
        output wire        ddr4_ddimm2_reset_n    , // .mem_reset_n
        output wire        ddr4_ddimm2_par        , // .mem_par
        input  wire        ddr4_ddimm2_alert_n    , // .mem_alert_n
        inout  wire [8:0]  ddr4_ddimm2_dqs_p      , // .mem_dqs
        inout  wire [8:0]  ddr4_ddimm2_dqs_n      , // .mem_dqs_n
        inout  wire [8:0]  ddr4_ddimm2_dbi_n      , // .mem_dbi_n
        inout  wire [17:9] ddr4_ddimm2_tdqs_n     , // .mem_tdqs_n
        inout  wire        fpga_dimm2_event_n    , // .mem_event_n
        inout  wire        fpga_dimm2_save_n     , // .mem_save_n
        inout  wire [71:0] ddr4_ddimm2_dq         , // .mem_dq
        input  wire        ddr4_ddimm2_rzq        , // .mem_rzq

        //PCIe 5.0 Genx16
        input  wire        fpga_pcie_perstn        , // PCIE_EP_PERSTN & PCIE_PERSTn
                                                     // PCIE_EP_PERSTN: from PCIe slot
                                                     // PCIE_PERSTn: Dedicated reset from on-board PB
        input  wire        pcie_ep_waken           , // WAKEN from PCIe slot, DNI by default
        output wire        pcie_1v2_clkreqn        , // PCIE_CLKREQn to PCIe slot, external pull-up

        input  wire        refclk_pcie_14c_ch0_p   , // PCIe reference clock #0 
        input  wire        refclk_pcie_14c_ch1_p   , // PCIe reference clock #1 

        input  wire        pcie_ep_rx_p0           , // PCIe RX channel #0
        input  wire        pcie_ep_rx_n0           , // PCIe RX channel #0
        input  wire        pcie_ep_rx_p1           , // PCIe RX channel #1
        input  wire        pcie_ep_rx_n1           , // PCIe RX channel #1
        input  wire        pcie_ep_rx_p2           , // PCIe RX channel #2
        input  wire        pcie_ep_rx_n2           , // PCIe RX channel #2
        input  wire        pcie_ep_rx_p3           , // PCIe RX channel #3
        input  wire        pcie_ep_rx_n3           , // PCIe RX channel #3
        input  wire        pcie_ep_rx_p4           , // PCIe RX channel #4
        input  wire        pcie_ep_rx_n4           , // PCIe RX channel #4
        input  wire        pcie_ep_rx_p5           , // PCIe RX channel #5
        input  wire        pcie_ep_rx_n5           , // PCIe RX channel #5
        input  wire        pcie_ep_rx_p6           , // PCIe RX channel #6
        input  wire        pcie_ep_rx_n6           , // PCIe RX channel #6
        input  wire        pcie_ep_rx_p7           , // PCIe RX channel #7
        input  wire        pcie_ep_rx_n7           , // PCIe RX channel #7
        input  wire        pcie_ep_rx_p8           , // PCIe RX channel #8
        input  wire        pcie_ep_rx_n8           , // PCIe RX channel #8
        input  wire        pcie_ep_rx_p9           , // PCIe RX channel #9
        input  wire        pcie_ep_rx_n9           , // PCIe RX channel #9
        input  wire        pcie_ep_rx_p10          , // PCIe RX channel #10
        input  wire        pcie_ep_rx_n10          , // PCIe RX channel #10
        input  wire        pcie_ep_rx_p11          , // PCIe RX channel #11
        input  wire        pcie_ep_rx_n11          , // PCIe RX channel #11
        input  wire        pcie_ep_rx_p12          , // PCIe RX channel #12
        input  wire        pcie_ep_rx_n12          , // PCIe RX channel #12
        input  wire        pcie_ep_rx_p13          , // PCIe RX channel #13
        input  wire        pcie_ep_rx_n13          , // PCIe RX channel #13
        input  wire        pcie_ep_rx_p14          , // PCIe RX channel #14
        input  wire        pcie_ep_rx_n14          , // PCIe RX channel #14
        input  wire        pcie_ep_rx_p15          , // PCIe RX channel #15
        input  wire        pcie_ep_rx_n15          , // PCIe RX channel #15

        output wire        pcie_ep_tx_p0           , // PCIe TX channel #0
        output wire        pcie_ep_tx_n0           , // PCIe TX channel #0
        output wire        pcie_ep_tx_p1           , // PCIe TX channel #1
        output wire        pcie_ep_tx_n1           , // PCIe TX channel #1
        output wire        pcie_ep_tx_p2           , // PCIe TX channel #2
        output wire        pcie_ep_tx_n2           , // PCIe TX channel #2
        output wire        pcie_ep_tx_p3           , // PCIe TX channel #3
        output wire        pcie_ep_tx_n3           , // PCIe TX channel #3
        output wire        pcie_ep_tx_p4           , // PCIe TX channel #4
        output wire        pcie_ep_tx_n4           , // PCIe TX channel #4
        output wire        pcie_ep_tx_p5           , // PCIe TX channel #5
        output wire        pcie_ep_tx_n5           , // PCIe TX channel #5
        output wire        pcie_ep_tx_p6           , // PCIe TX channel #6
        output wire        pcie_ep_tx_n6           , // PCIe TX channel #6
        output wire        pcie_ep_tx_p7           , // PCIe TX channel #7
        output wire        pcie_ep_tx_n7           , // PCIe TX channel #7
        output wire        pcie_ep_tx_p8           , // PCIe TX channel #8
        output wire        pcie_ep_tx_n8           , // PCIe TX channel #8
        output wire        pcie_ep_tx_p9           , // PCIe TX channel #9
        output wire        pcie_ep_tx_n9           , // PCIe TX channel #9
        output wire        pcie_ep_tx_p10          , // PCIe TX channel #10
        output wire        pcie_ep_tx_n10          , // PCIe TX channel #10
        output wire        pcie_ep_tx_p11          , // PCIe TX channel #11
        output wire        pcie_ep_tx_n11          , // PCIe TX channel #11
        output wire        pcie_ep_tx_p12          , // PCIe TX channel #12
        output wire        pcie_ep_tx_n12          , // PCIe TX channel #12
        output wire        pcie_ep_tx_p13          , // PCIe TX channel #13
        output wire        pcie_ep_tx_n13          , // PCIe TX channel #13
        output wire        pcie_ep_tx_p14          , // PCIe TX channel #14
        output wire        pcie_ep_tx_n14          , // PCIe TX channel #14
        output wire        pcie_ep_tx_p15          , // PCIe TX channel #15
        output wire        pcie_ep_tx_n15          , // PCIe TX channel #15

        //XCVR reference clock
        //input  wire        refclk_fgt12ach0_p    , // QSFPDD reference clock 100MHz from U22
        //input  wire        refclk_fgt12ach3_p    , // QSFPDD reference clock 153.6MHz from U22
        input  wire        refclk_fgt12ach4_p    , // QSFPDD reference clock 156.25MHz from U22
        //input  wire        refclk_fgt12ach5_p    , // QSFPDD reference clock 156.25MHz from U22
        //input  wire        refclk_fgt12ach6_p    , // QSFPDD reference clock 184.32MHz from U22

		//QSFPDD LEDs & control signals
        output wire [2:0]  qsfpdd0_fpga_led        , // QSFPDD0 LEDs
        output wire [2:0]  qsfpdd1_fpga_led        , // QSFPDD1 LEDs
        output  wire       qsfpdd_1v2_port_en      , // Enabling of I2C controller U14, external pull-up        
        input   wire       qsfpdd_1v2_port_int_n   , // Interrupt from I2C controller U14

        //QSFPDD XCVR channels
        input  wire [7:0]  qsfpdd0_rx_p          , // QSFPDD #0 RX P
        input  wire [7:0]  qsfpdd0_rx_n          , // QSFPDD #0 RX N
        output wire [7:0]  qsfpdd0_tx_p          , // QSFPDD #0 TX P
        output wire [7:0]  qsfpdd0_tx_n          ,  // QSFPDD #0 TX N
        input  wire [7:0]  qsfpdd1_rx_p          , // QSFPDD #1 RX P
        input  wire [7:0]  qsfpdd1_rx_n          , // QSFPDD #1 RX N
        output wire [7:0]  qsfpdd1_tx_p          , // QSFPDD #1 TX P
        output wire [7:0]  qsfpdd1_tx_n          , // QSFPDD #1 TX N

        //cxl
        input  wire        fpga_cxl_perstn         , // CXL_PERSTN
        output wire        fpga_cxl_rstn           , // CXL_PERSTN_O
        input  wire        fpga_cxl_prsntx1_n      , // CXL present x1 status
        input  wire        fpga_cxl_prsntx4_n      , // CXL present x4 status
        input  wire        fpga_cxl_prsntx8_n      , // CXL present x8 status
        input  wire        fpga_cxl_prsntx16_n     , // CXL present x16 status

        input  wire        refclk_cxl_15c_ch0_p    , // CXL reference clock #0
        input  wire        refclk_cxl_15c_ch1_p    , // CXL reference clock #1

        input  wire [15:0] cxl_rx_p                , // CXL RX channels 
        input  wire [15:0] cxl_rx_n                , // CXL RX channels 
        output wire [15:0] cxl_tx_p                , // CXL TX channels
        output wire [15:0] cxl_tx_n                , // CXL TX channels

		//Memory interface: DDR4 x72 comp #0
        input  wire        clk_ddr4_ch0_p        , // mem_ref_clk.clk
        output wire        ddr4_comp_ch0_clk_p   , //         mem.mem_ck
        output wire        ddr4_comp_ch0_clk_n   , //            .mem_ck_n
        output wire [16:0] ddr4_comp_ch0_a       , //            .mem_a
        output wire        ddr4_comp_ch0_act_n   , //            .mem_act_n
        output wire [1:0]  ddr4_comp_ch0_ba      , //            .mem_ba
        output wire [1:0]  ddr4_comp_ch0_bg      , //            .mem_bg
        output wire        ddr4_comp_ch0_cke     , //            .mem_cke
        output wire        ddr4_comp_ch0_cs_n    , //            .mem_cs_n
        output wire        ddr4_comp_ch0_odt     , //            .mem_odt
        output wire        ddr4_comp_ch0_reset_n , //            .mem_reset_n
        output wire        ddr4_comp_ch0_par     , //            .mem_par
        input  wire        ddr4_comp_ch0_alert_n , //            .mem_alert_n 
        inout  wire [8:0]  ddr4_comp_ch0_dqs_p   , //            .mem_dqs 
        inout  wire [8:0]  ddr4_comp_ch0_dqs_n   , //            .mem_dqs_n 
        inout  wire [8:0]  ddr4_comp_ch0_dbi_n   , //            .mem_dbi_n 
        inout  wire [71:0] ddr4_comp_ch0_dq      , //            .mem_dq
        input  wire        ddr4_comp_ch0_rzq     , //            .mem_rzq

        
		//Memory interface: DDR4 x72 comp #1
        input  wire        clk_ddr4_ch1_p        , // mem_ref_clk.clk
        output wire        ddr4_comp_ch1_clk_p   , //         mem.mem_ck
        output wire        ddr4_comp_ch1_clk_n   , //            .mem_ck_n
        output wire [16:0] ddr4_comp_ch1_a       , //            .mem_a
        output wire        ddr4_comp_ch1_act_n   , //            .mem_act_n
        output wire [1:0]  ddr4_comp_ch1_ba      , //            .mem_ba
        output wire [1:0]  ddr4_comp_ch1_bg      , //            .mem_bg
        output wire        ddr4_comp_ch1_cke     , //            .mem_cke
        output wire        ddr4_comp_ch1_cs_n    , //            .mem_cs_n
        output wire        ddr4_comp_ch1_odt     , //            .mem_odt
        output wire        ddr4_comp_ch1_reset_n , //            .mem_reset_n
        output wire        ddr4_comp_ch1_par     , //            .mem_par
        input  wire        ddr4_comp_ch1_alert_n , //            .mem_alert_n 
        inout  wire [8:0]  ddr4_comp_ch1_dqs_p   , //            .mem_dqs 
        inout  wire [8:0]  ddr4_comp_ch1_dqs_n   , //            .mem_dqs_n 
        inout  wire [8:0]  ddr4_comp_ch1_dbi_n   , //            .mem_dbi_n 
        inout  wire [71:0] ddr4_comp_ch1_dq      , //            .mem_dq
        input  wire        ddr4_comp_ch1_rzq     , //            .mem_rzq
		

        //HPS Peripherals
        input  wire        hps_osc_clk           , // HPS peripheral interfaces


        //SDM_IO
        output wire        FPGA_SDM_SCL          ,//SDM_IO0
        inout  wire        MSEL0                 ,//SDM_IO5
        inout  wire        MSEL1                 ,//SDM_IO7
        inout  wire        AVST8_READY           ,//SDM_IO8
        inout  wire        MSEL2                 ,//SDM_IO9
        inout  wire        AVST8_VALID           ,//SDM_IO11
        inout  wire        FPGA_SDM_SDA          ,//SDM_IO12
        inout  wire        AVST8_CLK             //SDM_IO14
//        inout  wire [7:0]  AVST8_DATA             //SDM_IO1,2,3,4,6,10,13,15

);

endmodule
