// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.

`timescale 1ns / 100ps
//`define SIMULATION
`define RTILE

module  ltssm_monitor
 (
    input   wire         i_reset_n                    , // reset
    input   wire         i_system_clk                 , // clock
    input   wire [2:0]   p0_tl_cfg_func_o             , //        p0_config_tl.tl_cfg_func            Check User Guide for details
    input   wire [4:0]   p0_tl_cfg_add_o              , //                    .tl_cfg_add             Check User Guide for details
    input   wire [15:0]  p0_tl_cfg_ctl_o              , //                    .tl_cfg_ctl             Check User Guide for details
    input   wire         p0_link_up_o                 , //       p0_power_mgnt.link_up                Check User Guide for details
    input   wire         p0_dl_up_o                   , //                    .dl_up                  Check User Guide for details
    input   wire         p0_surprise_down_err_o       , //                    .surprise_down_err      Check User Guide for details
    input   wire [2:0]   p0_pm_state_o                , //                    .pm_state               Check User Guide for details
    input   wire [5:0]   p0_ltssm_state_o             , //                    .ltssmstate             Check User Guide for details
    input   wire [31:0]  i_ltssm_cnt_ctrl             , // 
    output  reg  [31:0]  o_ltssm_monitor_code0        , // monitor code for error analysis
    output  wire [31:0]  o_ltssm_monitor_code1          // Power on interrupt, low active
        );

//#######################################################################################
//Parameters definition
//#######################################################################################
//PLDM over MCTP controller request FSM
localparam  IDLE                    = 8'h01; 
localparam  LINK_STRAT              = 8'h02; 
localparam  LINK_GOOD               = 8'h04; 
localparam  LTSSM_STATE_CHECK       = 8'h08; 
localparam  ERROR_HANDLING          = 8'h10; 
localparam  LTSSM_STATE_MONITOR     = 8'h20; 

`ifdef SIMULATION  		
localparam  LINK_STABLE_DURATION = 20'h01F;    // 31 x 20.48us = 634us, for simulation
`else
localparam  LINK_STABLE_DURATION = 20'h3FFFF;    // 262143 x 20.48us = 5.3s
`endif

//---------------------------------------------------------------------------------------
// Signal defination
//---------------------------------------------------------------------------------------
wire          slow_clk_pls_n         ; // Input/output
wire          slow_clk_pls_p         ; // Input/output
wire          link_good             ;
wire          link_good_out         ;
reg           link_good_out_reg     ;     
wire          link_good_out_reg_pos ;     
wire          link_good_out_neg     ;     

reg  [3:0]    link_speed_reg  ; // 
reg  [5:0]    link_width_reg  ; // 

reg  [3:0]    link_speed_reg1  ; // 
reg  [5:0]    link_width_reg1  ; // 
reg  [5:0]    p0_ltssm_state   ; // 

reg  [2:0]    ltssm_error_state_cnt  ; // 


reg  [7:0]    link_speed_transition_cnt;
reg  [15:0]   ltssm_state_transition_cnt;
reg  [7:0]    link_width_transition_cnt;


reg  [7:0]    fsm_curr_state;     // FSM current state
reg  [7:0]    fsm_next_state;     // FSM next state

reg           link_delay_cnt_clr;      
reg  [19:0]   link_delay_cnt    ;      
reg           delay_cnt_assert  ;    

//---------------------------------------------------------------------------------------
//Main function body
//---------------------------------------------------------------------------------------
//Inputs
assign link_good                 = p0_link_up_o & p0_dl_up_o;

//Outputs
assign o_ltssm_monitor_code1    = {ltssm_state_transition_cnt,link_width_transition_cnt,link_speed_transition_cnt};


//------------------------------------------------------------------------------------------
//Slow clock generation
slow_clocks         slow_clocks_inst0
                    (
                    .i_clk                     ( i_system_clk             ), // clock
                    .i_reset_n                 ( i_reset_n                ), // reset, low active
                    .o_clk_pls_n               (slow_clk_pls_n         ), // slow clock pulse, high active, negative phase
                    .o_clk_pls_p               (slow_clk_pls_p         )  // slow clock pulse, high active, positive phase
                    );                                                    // one system clock width, 20.48us/period



//---------------------------------------------------------------------------------------
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   link_good_out_reg <= 1'b0;
 else
   link_good_out_reg <= link_good_out;
end

assign link_good_out_reg_pos = link_good_out & (!link_good_out_reg); // Posedge of powerup_start
assign link_good_out_neg = (!link_good_out) & link_good_out_reg; // Negedge of powerup_done


//---------------------------------------------------------------------------------------
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   link_speed_reg <= 4'b0;
 `ifdef RTILE
 else
   link_speed_reg <= 4'h8;
 `else 
 else if(p0_tl_cfg_add_o == 5'h05)
   link_speed_reg <= p0_tl_cfg_ctl_o[15:12];
 `endif
end

//---------------------------------------------------------------------------------------
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   link_width_reg <= 6'b0;
 `ifdef RTILE  		
 else
   link_width_reg <= 6'h10;
 `else 
 else if(p0_tl_cfg_add_o == 5'h1c)
   link_width_reg <= p0_tl_cfg_ctl_o[5:0];
 `endif
end

always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
 begin
//   link_speed_reg1 <= 4'b0; 	
//   link_width_reg1 <= 6'b0;
   link_speed_reg1 <= 4'h8 ; 	//Gen4
   link_width_reg1 <= 6'h10;  //x16 
   p0_ltssm_state  <= 6'b0;
 end
 else
 begin
//   link_speed_reg1 <= link_speed_reg  ; 	
//   link_width_reg1 <= link_width_reg  ;
   link_speed_reg1 <= 4'h8 ; 	//Gen4
   link_width_reg1 <= 6'h10;  //x16 
   
   p0_ltssm_state  <= p0_ltssm_state_o;

 end
end


//---------------------------------------------------------------------------------------
//fpga_conf_done debounce: threshold around 5us
debouncer   #(
            .FILTER_WIDTH              (8                  ), // Debounce width, e.g. 4, 5, 6, 7...
            .INIT_BIT                  (1'b0               )  // High (1) or low (0)
            )
            debouncer_config_done
            (
            .i_rst_n                   ( i_reset_n            ), // Reset, low active
            .i_clk                     ( i_system_clk         ), // System clock
            .i_clk_div                 (8'd100                ), // System clock division factor
            .i_framein                 (link_good             ), // Frame input
            .o_frameout                (link_good_out         )  // Frame output after de-bouncing
            );

//#######################################################################################
//Power-on FSM, its related outputs and corner cases management
//#######################################################################################
//Finite State Machine current and next states
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   fsm_curr_state <= IDLE;
 else
   fsm_curr_state <= fsm_next_state;
end

//---------------------------------------------------------------------------------------
//Finite State Machine transformation control
//---------------------------------------------------------------------------------------
always @*
begin
  case(fsm_curr_state)
  IDLE:
    begin
      if(link_good_out_reg_pos)
        begin
        fsm_next_state            = LINK_STRAT;
        link_delay_cnt_clr        = 1'b1;
        end
      else
        begin
        fsm_next_state            = IDLE;
        link_delay_cnt_clr        = 1'b1;
        end
    end
    
  LINK_STRAT:
    begin
      if(link_good_out_neg)
        begin
        fsm_next_state            = IDLE;
        link_delay_cnt_clr        = 1'b0;
        end
      else if(link_good & slow_clk_pls_p & delay_cnt_assert)
        begin
        fsm_next_state            = LINK_GOOD;
        link_delay_cnt_clr        = 1'b0;
        end
      else
        begin
        fsm_next_state            = LINK_STRAT;
        link_delay_cnt_clr        = 1'b0;
        end
    end        
    
  LINK_GOOD:
    begin
      if( link_good_out_neg)
        begin
        fsm_next_state            = IDLE;
        link_delay_cnt_clr        = 1'b1;
        end
      else if(slow_clk_pls_p)
        begin
        fsm_next_state            = LTSSM_STATE_CHECK;
        link_delay_cnt_clr        = 1'b1;
        end
      else
        begin
        fsm_next_state            = LINK_GOOD;
        link_delay_cnt_clr        = 1'b1;
        end
    end
    
  LTSSM_STATE_CHECK:
    begin
      if( link_good_out_neg)
        begin
        fsm_next_state            = IDLE;
        link_delay_cnt_clr        = 1'b1;
        end
      else if((p0_ltssm_state_o==6'h11) & slow_clk_pls_p)
        begin
        fsm_next_state            = LTSSM_STATE_MONITOR;
        link_delay_cnt_clr        = 1'b1;
        end
      else if((p0_ltssm_state_o!=6'h11) & slow_clk_pls_p)
        begin
        	if(ltssm_error_state_cnt <= 3'd6)
           fsm_next_state            = LINK_STRAT;
          else
           fsm_next_state            = ERROR_HANDLING;

        link_delay_cnt_clr        = 1'b1;
        end
      else
        begin
        fsm_next_state            = LTSSM_STATE_CHECK;
        link_delay_cnt_clr        = 1'b1;
        end
    end

  ERROR_HANDLING:
    begin
      if( link_good_out_neg)
        begin
        fsm_next_state            = IDLE;
        link_delay_cnt_clr        = 1'b0;
        end
      else
        begin
        fsm_next_state           = ERROR_HANDLING;
        link_delay_cnt_clr       = 1'b0;
        end
    end
    
  LTSSM_STATE_MONITOR:
    begin
      if( link_good_out_neg)
        begin
        fsm_next_state             = IDLE;
        link_delay_cnt_clr         = 1'b0;
        end
      else
        begin
        fsm_next_state            = LTSSM_STATE_MONITOR;
        link_delay_cnt_clr        = 1'b0;
        end
    end          
    
  default:
    begin
    fsm_next_state            = IDLE;
    link_delay_cnt_clr        = 1'b0;
    end
  endcase
end

//###################################################################################################################
//Power-on FSM timeout control
//###################################################################################################################
//Power-on timeout
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   link_delay_cnt <= 20'd0;
 else if(link_delay_cnt_clr)
   link_delay_cnt <= 20'd0;
 else if(slow_clk_pls_p)
   begin
     if(link_delay_cnt == 20'd1048575) //1048575*20.48=20971500us=20s
       link_delay_cnt <= link_delay_cnt;
     else
       link_delay_cnt <= link_delay_cnt + 1'd1;
   end
 else
   link_delay_cnt <= link_delay_cnt;
end

always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   delay_cnt_assert <= 1'b0;
 else if(link_delay_cnt_clr)
   delay_cnt_assert <= 1'b0;
 else if((fsm_curr_state == LINK_STRAT) && (link_delay_cnt == LINK_STABLE_DURATION))
   delay_cnt_assert <= 1'b1;
 else
   delay_cnt_assert <= delay_cnt_assert;
end


//#######################################################################################
//Finite State Machine outputs control
//#######################################################################################
always @(posedge  i_system_clk or negedge  i_reset_n)
begin
 if(! i_reset_n)
   begin
   o_ltssm_monitor_code0           <= 32'b0;                            // Repower cycle, rising edge active
   link_speed_transition_cnt       <= 8'h0;                            // PFL II core power-on reset
   ltssm_state_transition_cnt      <= 16'h0;                            // FPGA reconfiguration via PFL II core
   link_width_transition_cnt       <= 8'h0;                            // FPGA page power-on selection, rising edge active
   ltssm_error_state_cnt           <= 3'b000;                          
   end
 else
   begin
     case(fsm_curr_state)
     IDLE:
       begin
       o_ltssm_monitor_code0           <= 32'b0;                        // Same as default setting
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_error_state_cnt           <= 3'b000;                       // Same as default setting
       end
       
     LINK_STRAT:
       begin
       o_ltssm_monitor_code0           <= 32'b0;                        // Same as default setting
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_error_state_cnt           <= ltssm_error_state_cnt;                       // Same as default setting
       end
       
     LINK_GOOD:
       begin
       o_ltssm_monitor_code0           <= 32'b0;                        // Same as default setting
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       if(slow_clk_pls_p)
       ltssm_error_state_cnt           <= ltssm_error_state_cnt + 3'd1;                       // Same as default setting
       else
       ltssm_error_state_cnt           <= ltssm_error_state_cnt;                       // Same as default setting

       end
       
     LTSSM_STATE_CHECK:
       begin
       o_ltssm_monitor_code0           <= 32'b0;                        // Same as default setting
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_error_state_cnt           <= ltssm_error_state_cnt;                       // Same as default setting
       end

       
     ERROR_HANDLING:
       begin
       o_ltssm_monitor_code0          <= {16'h00f,16'hffff};               // Keep as it is
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_error_state_cnt           <= ltssm_error_state_cnt;                       // Same as default setting
       end
       
     LTSSM_STATE_MONITOR:
       begin
       o_ltssm_monitor_code0           <= {12'h01,2'b0,p0_ltssm_state_o,2'b0,link_width_reg,link_speed_reg};               // Keep as it is
       
       if(i_ltssm_cnt_ctrl[2])
         ltssm_state_transition_cnt      <= 16'h0;            // Keep as it is
       else if((p0_ltssm_state==6'h11) && (p0_ltssm_state_o ==6'h0D))
         ltssm_state_transition_cnt      <= ltssm_state_transition_cnt + 16'h1;            // Keep as it is
       else
         ltssm_state_transition_cnt      <= ltssm_state_transition_cnt;            // Keep as it is

       if(i_ltssm_cnt_ctrl[1])
         link_width_transition_cnt      <= 8'h0;            // Keep as it is
 `ifdef RTILE
 `else 
       else if((link_width_reg!=link_width_reg1))
         link_width_transition_cnt      <= link_width_transition_cnt + 8'h1;            // Keep as it is
 `endif
       else
         link_width_transition_cnt      <= link_width_transition_cnt;            // Keep as it is

       if(i_ltssm_cnt_ctrl[0])
         link_speed_transition_cnt      <= 8'h0;            // Keep as it is
 `ifdef RTILE
 `else 
       else if((link_speed_reg!=link_speed_reg1))
         link_speed_transition_cnt      <= link_speed_transition_cnt + 8'h1;            // Keep as it is
 `endif
       else
         link_speed_transition_cnt      <= link_speed_transition_cnt;            // Keep as it is

       ltssm_error_state_cnt <= ltssm_error_state_cnt;       // Keep as it is
       
       end
       
     default:
       begin
       o_ltssm_monitor_code0           <= 32'b0;                        // Same as default setting
       link_speed_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_state_transition_cnt      <= 16'h0;                        // Same as default setting
       link_width_transition_cnt       <= 8'h0;                         // Same as default setting
       ltssm_error_state_cnt           <= 3'b000;                       // Same as default setting
       end
     endcase
   end
end

endmodule


//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------