// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.

`timescale 1ns / 100ps

module  reset_generator
        #(
        parameter     RESET_COUNTER = 1024  // <=65535
        )
        (
        input   wire  i_clk               , // input clock
        input   wire  i_reset_n           , // input reset, low active
        input   wire  i_resetn_debug      , // input reset debug, low active
        output  wire  o_reset_n             // output reset, low active
        );

//--------------------------------------------------
// Signal definition
//--------------------------------------------------
wire            clk;
wire            reset_n_in;
wire            resetn_debug;
reg             reset_n_out;

reg     [9:0]   ctr;
reg             slow_clk;
reg             slow_clk_reg;
wire            slow_clk_pos;
reg     [15:0]  reset_cnt;

assign  clk          = i_clk;
assign  reset_n_in   = i_reset_n;
assign  resetn_debug = i_resetn_debug;
assign  o_reset_n    = reset_n_out;

//--------------------------------------------------
// Main function body
//--------------------------------------------------
always @(posedge clk or negedge reset_n_in)
begin
  if(!reset_n_in)
     begin
     ctr      <= 10'd0;
     slow_clk <= 1'b0;
     end
  else if(ctr == 10'd1023)
     begin
     ctr      <= 10'd0;
     slow_clk <= !slow_clk;
     end
  else if(ctr == 10'd511)
     begin
     ctr      <= ctr + 1'b1;
     slow_clk <= !slow_clk;
     end
  else
     ctr <= ctr + 1'b1;
end

always @(posedge clk or negedge reset_n_in)
begin
  if(!reset_n_in)
     slow_clk_reg <= 1'b0;
  else
     slow_clk_reg <= slow_clk;
end

assign slow_clk_pos = slow_clk & (!slow_clk_reg); // posedge of slow_clk

always @(posedge clk or negedge reset_n_in)
begin
  if(!reset_n_in)
     reset_cnt <= 16'd0;
  else if(slow_clk_pos)
    begin
    if (reset_cnt == RESET_COUNTER)
        reset_cnt <= reset_cnt;
    else
        reset_cnt <= reset_cnt + 1'b1;
    end
  else
     reset_cnt <= reset_cnt;
end

always @(posedge clk or negedge reset_n_in)
begin
  if(!reset_n_in)
     reset_n_out <= 1'b0;
  else if(resetn_debug == 1'b0)
     reset_n_out <= 1'b0;
  else if(slow_clk_pos)
    begin
      if(reset_cnt == RESET_COUNTER)
         reset_n_out <= 1'b1;
      else
         reset_n_out <= reset_n_out;
    end
  else
     reset_n_out <= reset_n_out;
end

endmodule
//------------------------------------------------------------------------------
// end of file
//------------------------------------------------------------------------------