// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns / 100ps

module  pcie_ep (
        //Reset&Clock
        input  wire        cpu_1v2_resetn          , // Reset, low active
		input  wire        clk_sys_100m_p          , //System clock, 100MHz
        //input  wire        clk_sys_bak_50m_p       , // System clock, 50MHz

        //I2C interfaces
        output wire        fpga_i2c_en             , // FPGA I2C interface #1 enable, DIPSW ctrl
        inout  wire        fpga_i2c1_scl           , // FPGA I2C interface #1 SCL
        inout  wire        fpga_i2c1_sda           , // FPGA I2C interface #1 SDA
        output wire        fpga_i2c2_en            , // FPGA I2C interface #2 enable, DIPSW ctrl
        inout  wire        fpga_i2c2_scl           , // FPGA I2C interface #2 SCL
        inout  wire        fpga_i2c2_sda           , // FPGA I2C interface #2 SDA
        inout  wire        fpga_dimm_scl           , // FPGA I2C interface dedicated for RDIMM access
        inout  wire        fpga_dimm_sda           , // FPGA I2C interface dedicated for RDIMM access

        //GPIOs
        output wire [3:0]  fpga_led                , // FPGA user LEDs
      //input  wire        gxf_perstn              , // Dedicated reset from on-board PB, currently not used

        //PCIe 5.0 Genx16
        input  wire        fpga_pcie_perstn        , // PCIE_EP_PERSTN & PCIE_PERSTn
                                                     // PCIE_EP_PERSTN: from PCIe slot
                                                     // PCIE_PERSTn: Dedicated reset from on-board PB
        input  wire        pcie_ep_waken           , // WAKEN from PCIe slot, DNI by default
        output wire        pcie_1v2_clkreqn        , // PCIE_CLKREQn to PCIe slot, external pull-up

        input  wire        refclk_pcie_14c_ch0_p   , // PCIe reference clock #0 
        input  wire        refclk_pcie_14c_ch1_p   , // PCIe reference clock #1 

        input  wire        pcie_ep_rx_p0           , // PCIe RX channel #0
        input  wire        pcie_ep_rx_n0           , // PCIe RX channel #0
        input  wire        pcie_ep_rx_p1           , // PCIe RX channel #1
        input  wire        pcie_ep_rx_n1           , // PCIe RX channel #1
        input  wire        pcie_ep_rx_p2           , // PCIe RX channel #2
        input  wire        pcie_ep_rx_n2           , // PCIe RX channel #2
        input  wire        pcie_ep_rx_p3           , // PCIe RX channel #3
        input  wire        pcie_ep_rx_n3           , // PCIe RX channel #3
        input  wire        pcie_ep_rx_p4           , // PCIe RX channel #4
        input  wire        pcie_ep_rx_n4           , // PCIe RX channel #4
        input  wire        pcie_ep_rx_p5           , // PCIe RX channel #5
        input  wire        pcie_ep_rx_n5           , // PCIe RX channel #5
        input  wire        pcie_ep_rx_p6           , // PCIe RX channel #6
        input  wire        pcie_ep_rx_n6           , // PCIe RX channel #6
        input  wire        pcie_ep_rx_p7           , // PCIe RX channel #7
        input  wire        pcie_ep_rx_n7           , // PCIe RX channel #7
        input  wire        pcie_ep_rx_p8           , // PCIe RX channel #8
        input  wire        pcie_ep_rx_n8           , // PCIe RX channel #8
        input  wire        pcie_ep_rx_p9           , // PCIe RX channel #9
        input  wire        pcie_ep_rx_n9           , // PCIe RX channel #9
        input  wire        pcie_ep_rx_p10          , // PCIe RX channel #10
        input  wire        pcie_ep_rx_n10          , // PCIe RX channel #10
        input  wire        pcie_ep_rx_p11          , // PCIe RX channel #11
        input  wire        pcie_ep_rx_n11          , // PCIe RX channel #11
        input  wire        pcie_ep_rx_p12          , // PCIe RX channel #12
        input  wire        pcie_ep_rx_n12          , // PCIe RX channel #12
        input  wire        pcie_ep_rx_p13          , // PCIe RX channel #13
        input  wire        pcie_ep_rx_n13          , // PCIe RX channel #13
        input  wire        pcie_ep_rx_p14          , // PCIe RX channel #14
        input  wire        pcie_ep_rx_n14          , // PCIe RX channel #14
        input  wire        pcie_ep_rx_p15          , // PCIe RX channel #15
        input  wire        pcie_ep_rx_n15          , // PCIe RX channel #15

        output wire        pcie_ep_tx_p0           , // PCIe TX channel #0
        output wire        pcie_ep_tx_n0           , // PCIe TX channel #0
        output wire        pcie_ep_tx_p1           , // PCIe TX channel #1
        output wire        pcie_ep_tx_n1           , // PCIe TX channel #1
        output wire        pcie_ep_tx_p2           , // PCIe TX channel #2
        output wire        pcie_ep_tx_n2           , // PCIe TX channel #2
        output wire        pcie_ep_tx_p3           , // PCIe TX channel #3
        output wire        pcie_ep_tx_n3           , // PCIe TX channel #3
        output wire        pcie_ep_tx_p4           , // PCIe TX channel #4
        output wire        pcie_ep_tx_n4           , // PCIe TX channel #4
        output wire        pcie_ep_tx_p5           , // PCIe TX channel #5
        output wire        pcie_ep_tx_n5           , // PCIe TX channel #5
        output wire        pcie_ep_tx_p6           , // PCIe TX channel #6
        output wire        pcie_ep_tx_n6           , // PCIe TX channel #6
        output wire        pcie_ep_tx_p7           , // PCIe TX channel #7
        output wire        pcie_ep_tx_n7           , // PCIe TX channel #7
        output wire        pcie_ep_tx_p8           , // PCIe TX channel #8
        output wire        pcie_ep_tx_n8           , // PCIe TX channel #8
        output wire        pcie_ep_tx_p9           , // PCIe TX channel #9
        output wire        pcie_ep_tx_n9           , // PCIe TX channel #9
        output wire        pcie_ep_tx_p10          , // PCIe TX channel #10
        output wire        pcie_ep_tx_n10          , // PCIe TX channel #10
        output wire        pcie_ep_tx_p11          , // PCIe TX channel #11
        output wire        pcie_ep_tx_n11          , // PCIe TX channel #11
        output wire        pcie_ep_tx_p12          , // PCIe TX channel #12
        output wire        pcie_ep_tx_n12          , // PCIe TX channel #12
        output wire        pcie_ep_tx_p13          , // PCIe TX channel #13
        output wire        pcie_ep_tx_n13          , // PCIe TX channel #13
        output wire        pcie_ep_tx_p14          , // PCIe TX channel #14
        output wire        pcie_ep_tx_n14          , // PCIe TX channel #14
        output wire        pcie_ep_tx_p15          , // PCIe TX channel #15
        output wire        pcie_ep_tx_n15            // PCIe TX channel #15
        );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Signal definitions
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
wire          ninit_done/* synthesis keep */;
wire          global_resetn/* synthesis keep */;
reg  [26:0]   heart_beat_cnt/* synthesis keep */;
reg  [26:0]   heart_beat_cnt1/* synthesis keep */;
wire          hip_out_clk_clk                           ;//         hip_out_clk.clk 
wire          pin_perst_pin_perst_o                     ;//                    .pin_perst_o
wire          dut_p0_hip_status_link_up                 ;//   dut_p0_hip_status.link_up
wire          dut_p0_hip_status_dl_up                   ;//                    .dl_up
wire          dut_p0_hip_status_surprise_down_err       ;//                    .surprise_down_err
wire          dut_p0_hip_status_dl_timer_update         ;//                    .dl_timer_update
wire [5:0]    dut_p0_hip_status_ltssm_state_delay       ;//                    .ltssm_state_delay
wire          dut_p0_hip_status_ltssm_st_hipfifo_ovrflw ;//                    .ltssm_st_hipfifo_ovrflw
wire [7:0]    dut_p0_power_mgnt_pm_curnt_state          ;//   dut_p0_power_mgnt.pm_curnt_state
wire [31:0]   dut_p0_power_mgnt_pm_dstate               ;//                    .pm_dstate
wire [7:0]    dut_p0_power_mgnt_app_req_retry_en        ;//                    .app_req_retry_en
wire          dut_p0_power_mgnt_app_xfer_pending        ;//                    .app_xfer_pending
wire          dut_p0_power_mgnt_apps_ready_entr_l23     ;//                    .apps_ready_entr_l23
wire [7:0]    dut_p0_power_mgnt_apps_pm_xmt_pme         ;//                    .apps_pm_xmt_pme
wire [7:0]    dut_p0_pld_gp_status                      ;//       dut_p0_pld_gp.status
wire [7:0]    dut_p0_pld_gp_ctrl                        ;//                    .ctrl
wire          dut_p0_pld_gp_status_ready                ;//                    .status_ready
wire [7:0]    dut_p0_hip_reconfig_readdata              ;// dut_p0_hip_reconfig.readdata
wire          dut_p0_hip_reconfig_readdatavalid         ;//                    .readdatavalid
wire          dut_p0_hip_reconfig_waitrequest           ;//                    .waitrequest
wire [31:0]   dut_p0_hip_reconfig_address               ;//                    .address
wire          dut_p0_hip_reconfig_write                 ;//                    .write
wire [7:0]    dut_p0_hip_reconfig_writedata             ;//                    .writedata
wire          dut_p0_hip_reconfig_read                  ;//                    .read

wire [31:0]  i_ltssm_cnt_ctrl            ;       
wire [31:0]  o_ltssm_monitor_code0       ;       
wire [31:0]  o_ltssm_monitor_code1       ;   
wire         outclk0_100m                ;
wire         outclk1_50m                 ;
wire          iopll_lock       /* synthesis keep */;
wire          global_resetn1   /* synthesis keep */;


assign dut_p0_power_mgnt_app_req_retry_en    = 8'h0  ;    //                    .app_req_retry_en
assign dut_p0_power_mgnt_app_xfer_pending    = 1'h0  ;    //                    .app_xfer_pending
assign dut_p0_power_mgnt_apps_ready_entr_l23 = 1'h0  ;    //                    .apps_ready_entr_l23
assign dut_p0_power_mgnt_apps_pm_xmt_pme     = 8'h0  ;    //                    .apps_pm_xmt_pme
assign dut_p0_pld_gp_status                  = 8'h0  ;    //       dut_p0_pld_gp.status

assign dut_p0_hip_reconfig_address           = 32'h0 ;    //                    .address
assign dut_p0_hip_reconfig_write             = 1'h0  ;    //                    .write
assign dut_p0_hip_reconfig_writedata         = 8'h0  ;    //                    .writedata
assign dut_p0_hip_reconfig_read              = 1'h0  ;    //                    .read


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Input/output assignments
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
assign fpga_i2c_en      = 1'bz;
assign fpga_i2c2_en     = 1'bz;
assign fpga_i2c1_scl    = 1'bz;
assign fpga_i2c1_sda    = 1'bz;
assign fpga_i2c2_scl    = 1'bz;
assign fpga_i2c2_sda    = 1'bz;
assign fpga_dimm_scl    = 1'bz;
assign fpga_dimm_sda    = 1'bz;

assign fpga_led[3:0]    = {heart_beat_cnt[26], heart_beat_cnt[25], heart_beat_cnt[24],heart_beat_cnt[26]};
assign pcie_1v2_clkreqn = 1'bz;

assign global_resetn = !ninit_done & cpu_1v2_resetn;
assign global_resetn1 = global_resetn & iopll_lock;
//assign global_resetn = cpu_1v2_resetn;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//iopll
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
iopll    iopll_0 (
		.rst      (!global_resetn),      //   input,  width = 1,   reset.reset
		.refclk   (clk_sys_100m_p),   //   input,  width = 1,  refclk.clk 100m
		.locked   (iopll_lock),   //  output,  width = 1,  locked.export
		.outclk_0 (outclk0_100m), //  output,  width = 1, outclk0.clk 100m
		.outclk_1 (outclk1_50m)  //  output,  width = 1, outclk1.clk  50m
	);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Heart-beat indicate
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @(posedge outclk1_50m or negedge global_resetn1)
    if (!global_resetn1)
        heart_beat_cnt <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt <= heart_beat_cnt + 1'b1;

/* always @(posedge refclk_pcie_14c_ch0_p or negedge global_resetn1)
    if (!global_resetn1)
        heart_beat_cnt1 <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt1 <= heart_beat_cnt1 + 1'b1;
 */


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Example design
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
pcie_ed     pcie_ed_inst0
            (
            .refclk0_clk                               (refclk_pcie_14c_ch0_p                     ), //    refclk0.clk
            .refclk1_clk                               (refclk_pcie_14c_ch1_p                     ), //    refclk1.clk
            .pin_perst_reset_n                       (fpga_pcie_perstn                          ), //  pin_perst.pin_perst
            .dut_ninit_done_reset                      (ninit_done                                ), //      dut_ninit_done.ninit_done
            .resetip_ninit_done_reset                  (ninit_done                                ), //  resetip_ninit_done.ninit_done
            .hipclk_out_clk_clk                        (hip_out_clk_clk                           ), //         hip_out_clk.clk
          //.pin_perst_pin_perst_o                     (pin_perst_pin_perst_o                     ), //                    .pin_perst_o              
            .dut_p0_hip_status_link_up                 (dut_p0_hip_status_link_up                 ), //   dut_p0_hip_status.link_up                  
            .dut_p0_hip_status_dl_up                   (dut_p0_hip_status_dl_up                   ), //                    .dl_up                    
            .dut_p0_hip_status_surprise_down_err       (dut_p0_hip_status_surprise_down_err       ), //                    .surprise_down_err        
            .dut_p0_hip_status_dl_timer_update         (dut_p0_hip_status_dl_timer_update         ), //                    .dl_timer_update          
            .dut_p0_hip_status_ltssm_state_delay       (dut_p0_hip_status_ltssm_state_delay       ), //                    .ltssm_state_delay        
            .dut_p0_hip_status_ltssm_st_hipfifo_ovrflw (dut_p0_hip_status_ltssm_st_hipfifo_ovrflw ), //                    .ltssm_st_hipfifo_ovrflw  
            .dut_p0_power_mgnt_pm_curnt_state          (dut_p0_power_mgnt_pm_curnt_state          ), //   dut_p0_power_mgnt.pm_curnt_state           
            .dut_p0_power_mgnt_pm_dstate               (dut_p0_power_mgnt_pm_dstate               ), //                    .pm_dstate                
            .dut_p0_power_mgnt_app_req_retry_en        (dut_p0_power_mgnt_app_req_retry_en        ), //                    .app_req_retry_en         
            .dut_p0_power_mgnt_app_xfer_pending        (dut_p0_power_mgnt_app_xfer_pending        ), //                    .app_xfer_pending         
            .dut_p0_power_mgnt_apps_ready_entr_l23     (dut_p0_power_mgnt_apps_ready_entr_l23     ), //                    .apps_ready_entr_l23      
            .dut_p0_power_mgnt_apps_pm_xmt_pme         (dut_p0_power_mgnt_apps_pm_xmt_pme         ), //                    .apps_pm_xmt_pme          
            .dut_p0_pld_gp_status                      (dut_p0_pld_gp_status                      ), //       dut_p0_pld_gp.status                   
            .dut_p0_pld_gp_ctrl                        (dut_p0_pld_gp_ctrl                        ), //                    .ctrl                     
            .dut_p0_pld_gp_status_ready                (dut_p0_pld_gp_status_ready                ), //                    .status_ready             
            .dut_p0_hip_reconfig_readdata              (dut_p0_hip_reconfig_readdata              ), // dut_p0_hip_reconfig.readdata                 
            .dut_p0_hip_reconfig_readdatavalid         (dut_p0_hip_reconfig_readdatavalid         ), //                    .readdatavalid            
            .dut_p0_hip_reconfig_waitrequest           (dut_p0_hip_reconfig_waitrequest           ), //                    .waitrequest              
            .dut_p0_hip_reconfig_address               (dut_p0_hip_reconfig_address               ), //                    .address                  
            .dut_p0_hip_reconfig_write                 (dut_p0_hip_reconfig_write                 ), //                    .write                    
            .dut_p0_hip_reconfig_writedata             (dut_p0_hip_reconfig_writedata             ), //                    .writedata                
            .dut_p0_hip_reconfig_read                  (dut_p0_hip_reconfig_read                  ), //                    .read                     
            .hip_serial_rx_n_in0                       (pcie_ep_rx_n0                             ), // hip_serial.rx_n_in0
            .hip_serial_rx_n_in1                       (pcie_ep_rx_n1                             ), //           .rx_n_in1
            .hip_serial_rx_n_in2                       (pcie_ep_rx_n2                             ), //           .rx_n_in2
            .hip_serial_rx_n_in3                       (pcie_ep_rx_n3                             ), //           .rx_n_in3
            .hip_serial_rx_n_in4                       (pcie_ep_rx_n4                             ), //           .rx_n_in4
            .hip_serial_rx_n_in5                       (pcie_ep_rx_n5                             ), //           .rx_n_in5
            .hip_serial_rx_n_in6                       (pcie_ep_rx_n6                             ), //           .rx_n_in6
            .hip_serial_rx_n_in7                       (pcie_ep_rx_n7                             ), //           .rx_n_in7
            .hip_serial_rx_n_in8                       (pcie_ep_rx_n8                             ), //           .rx_n_in8
            .hip_serial_rx_n_in9                       (pcie_ep_rx_n9                             ), //           .rx_n_in9
            .hip_serial_rx_n_in10                      (pcie_ep_rx_n10                            ), //           .rx_n_in10
            .hip_serial_rx_n_in11                      (pcie_ep_rx_n11                            ), //           .rx_n_in11
            .hip_serial_rx_n_in12                      (pcie_ep_rx_n12                            ), //           .rx_n_in12
            .hip_serial_rx_n_in13                      (pcie_ep_rx_n13                            ), //           .rx_n_in13
            .hip_serial_rx_n_in14                      (pcie_ep_rx_n14                            ), //           .rx_n_in14
            .hip_serial_rx_n_in15                      (pcie_ep_rx_n15                            ), //           .rx_n_in15
            .hip_serial_rx_p_in0                       (pcie_ep_rx_p0                             ), //           .rx_p_in0
            .hip_serial_rx_p_in1                       (pcie_ep_rx_p1                             ), //           .rx_p_in1
            .hip_serial_rx_p_in2                       (pcie_ep_rx_p2                             ), //           .rx_p_in2
            .hip_serial_rx_p_in3                       (pcie_ep_rx_p3                             ), //           .rx_p_in3
            .hip_serial_rx_p_in4                       (pcie_ep_rx_p4                             ), //           .rx_p_in4
            .hip_serial_rx_p_in5                       (pcie_ep_rx_p5                             ), //           .rx_p_in5
            .hip_serial_rx_p_in6                       (pcie_ep_rx_p6                             ), //           .rx_p_in6
            .hip_serial_rx_p_in7                       (pcie_ep_rx_p7                             ), //           .rx_p_in7
            .hip_serial_rx_p_in8                       (pcie_ep_rx_p8                             ), //           .rx_p_in8
            .hip_serial_rx_p_in9                       (pcie_ep_rx_p9                             ), //           .rx_p_in9
            .hip_serial_rx_p_in10                      (pcie_ep_rx_p10                            ), //           .rx_p_in10
            .hip_serial_rx_p_in11                      (pcie_ep_rx_p11                            ), //           .rx_p_in11
            .hip_serial_rx_p_in12                      (pcie_ep_rx_p12                            ), //           .rx_p_in12
            .hip_serial_rx_p_in13                      (pcie_ep_rx_p13                            ), //           .rx_p_in13
            .hip_serial_rx_p_in14                      (pcie_ep_rx_p14                            ), //           .rx_p_in14
            .hip_serial_rx_p_in15                      (pcie_ep_rx_p15                            ), //           .rx_p_in15
            .hip_serial_tx_n_out0                      (pcie_ep_tx_n0                             ), //           .tx_n_out0
            .hip_serial_tx_n_out1                      (pcie_ep_tx_n1                             ), //           .tx_n_out1
            .hip_serial_tx_n_out2                      (pcie_ep_tx_n2                             ), //           .tx_n_out2
            .hip_serial_tx_n_out3                      (pcie_ep_tx_n3                             ), //           .tx_n_out3
            .hip_serial_tx_n_out4                      (pcie_ep_tx_n4                             ), //           .tx_n_out4
            .hip_serial_tx_n_out5                      (pcie_ep_tx_n5                             ), //           .tx_n_out5
            .hip_serial_tx_n_out6                      (pcie_ep_tx_n6                             ), //           .tx_n_out6
            .hip_serial_tx_n_out7                      (pcie_ep_tx_n7                             ), //           .tx_n_out7
            .hip_serial_tx_n_out8                      (pcie_ep_tx_n8                             ), //           .tx_n_out8
            .hip_serial_tx_n_out9                      (pcie_ep_tx_n9                             ), //           .tx_n_out9
            .hip_serial_tx_n_out10                     (pcie_ep_tx_n10                            ), //           .tx_n_out10
            .hip_serial_tx_n_out11                     (pcie_ep_tx_n11                            ), //           .tx_n_out11
            .hip_serial_tx_n_out12                     (pcie_ep_tx_n12                            ), //           .tx_n_out12
            .hip_serial_tx_n_out13                     (pcie_ep_tx_n13                            ), //           .tx_n_out13
            .hip_serial_tx_n_out14                     (pcie_ep_tx_n14                            ), //           .tx_n_out14
            .hip_serial_tx_n_out15                     (pcie_ep_tx_n15                            ), //           .tx_n_out15
            .hip_serial_tx_p_out0                      (pcie_ep_tx_p0                             ), //           .tx_p_out0
            .hip_serial_tx_p_out1                      (pcie_ep_tx_p1                             ), //           .tx_p_out1
            .hip_serial_tx_p_out2                      (pcie_ep_tx_p2                             ), //           .tx_p_out2
            .hip_serial_tx_p_out3                      (pcie_ep_tx_p3                             ), //           .tx_p_out3
            .hip_serial_tx_p_out4                      (pcie_ep_tx_p4                             ), //           .tx_p_out4
            .hip_serial_tx_p_out5                      (pcie_ep_tx_p5                             ), //           .tx_p_out5
            .hip_serial_tx_p_out6                      (pcie_ep_tx_p6                             ), //           .tx_p_out6
            .hip_serial_tx_p_out7                      (pcie_ep_tx_p7                             ), //           .tx_p_out7
            .hip_serial_tx_p_out8                      (pcie_ep_tx_p8                             ), //           .tx_p_out8
            .hip_serial_tx_p_out9                      (pcie_ep_tx_p9                             ), //           .tx_p_out9
            .hip_serial_tx_p_out10                     (pcie_ep_tx_p10                            ), //           .tx_p_out10
            .hip_serial_tx_p_out11                     (pcie_ep_tx_p11                            ), //           .tx_p_out11
            .hip_serial_tx_p_out12                     (pcie_ep_tx_p12                            ), //           .tx_p_out12
            .hip_serial_tx_p_out13                     (pcie_ep_tx_p13                            ), //           .tx_p_out13
            .hip_serial_tx_p_out14                     (pcie_ep_tx_p14                            ), //           .tx_p_out14
            .hip_serial_tx_p_out15                     (pcie_ep_tx_p15                            )  //           .tx_p_out15
            );


ltssm_monitor  ltssm_monitor_i(
    .i_reset_n                    (global_resetn1                       ), 
    .i_system_clk                 (hip_out_clk_clk                      ), 
    .p0_tl_cfg_func_o             (3'h0                                 ), 
    .p0_tl_cfg_add_o              (5'h0                                 ), 
    .p0_tl_cfg_ctl_o              (16'h0                                ), 
    .p0_link_up_o                 (dut_p0_hip_status_link_up            ), 
    .p0_dl_up_o                   (dut_p0_hip_status_dl_up              ), 
    .p0_surprise_down_err_o       (dut_p0_hip_status_surprise_down_err  ), 
    .p0_pm_state_o                (3'h0                                 ), 
    .p0_ltssm_state_o             (dut_p0_hip_status_ltssm_state_delay  ), 
    .i_ltssm_cnt_ctrl             (i_ltssm_cnt_ctrl                     ), 
    .o_ltssm_monitor_code0        (o_ltssm_monitor_code0                ), 
    .o_ltssm_monitor_code1        (o_ltssm_monitor_code1                )
 );
	   
pcie_test 	 pcie_test_i(   
    .clk_50_clk                                 (outclk1_50m     ),                               
    .reset_50_reset_n                           (global_resetn1        ),                         
    .pio_0_external_test_control_config_export  (i_ltssm_cnt_ctrl      ),
    .pio_1_external_test_status_state0_export   (o_ltssm_monitor_code0 ), 
    .pio_2_external_test_status_state1_export   (o_ltssm_monitor_code1 ),
    .pio_3_external_ltssm_monitor_export        ( )
    );
	   
	   


endmodule
	