-- (C) 2001-2024 Intel Corporation. All rights reserved.
-- Your use of Intel Corporation's design tools, logic functions and other 
-- software and tools, and its AMPP partner logic functions, and any output 
-- files from any of the foregoing (including device programming or simulation 
-- files), and any associated documentation or information are expressly subject 
-- to the terms and conditions of the Intel Program License Subscription 
-- Agreement, Intel FPGA IP License Agreement, or other applicable 
-- license agreement, including, without limitation, that your use is for the 
-- sole purpose of programming logic devices manufactured by Intel and sold by 
-- Intel or its authorized distributors.  Please refer to the applicable 
-- agreement for further details.


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_UNSIGNED.all;

entity  temp_info is
        port (
        clk                  : in  std_logic;                     -- clock
        reset_n              : in  std_logic;                     -- reset, low active
        chipselect_n         : in  std_logic;                     -- chip select signal
        read_n               : in  std_logic;                     -- output enable
        av_address           : in  std_logic_vector(3 downto 0);  -- address
        av_data_read         : out std_logic_vector(31 downto 0); -- read data
        fpga_core_temp       : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile0_temp : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile1_temp : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile2_temp : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile3_temp : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile4_temp : in std_logic_vector (31 downto 0); -- 
        fpga_xcvr_tile5_temp : in std_logic_vector (31 downto 0); -- 
        fpga_max_temp        : in std_logic_vector (31 downto 0)  -- 
        );
end temp_info;

architecture rtl of temp_info is

constant SOF:     std_logic_vector(31 downto 0):=x"4750494F"; -- GPIO project
constant VERSION: std_logic_vector(31 downto 0):=x"00000001"; -- Build version

begin

process(reset_n, clk)begin
    if(reset_n = '0')then
       av_data_read <= (others => '0');
    elsif(clk'event and clk = '1')then
        if(chipselect_n = '0' and read_n = '0')then
            case av_address is
              when "0000" => av_data_read <= SOF;
              when "0001" => av_data_read <= VERSION;
              when "0010" => av_data_read <= (others => '0');
              when "0011" => av_data_read <= (others => '0');
              when "0100" => av_data_read <= (others => '0');
              when "0101" => av_data_read <= (others => '0');
              when "0110" => av_data_read <= (others => '0');
              when "0111" => av_data_read <= (others => '0');
              when "1000" => av_data_read <= fpga_core_temp;
              when "1001" => av_data_read <= fpga_xcvr_tile0_temp;
              when "1010" => av_data_read <= fpga_xcvr_tile1_temp;
              when "1011" => av_data_read <= fpga_xcvr_tile2_temp;
              when "1100" => av_data_read <= fpga_xcvr_tile3_temp;
              when "1101" => av_data_read <= fpga_xcvr_tile4_temp;
              when "1110" => av_data_read <= fpga_xcvr_tile5_temp;
              when "1111" => av_data_read <= fpga_max_temp;
              when others => av_data_read <= (others => '0');
            end case;
        end if;
    end if;
end process;

end rtl;

