// (C) 2001-2024 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ns / 100ps

module  qsfpdd_xcvr (
        //Reset&Clock
        input  wire        cpu_1v2_resetn        , // Reset, low active
        input  wire        clk_sys_100m_p        , // System clock, 100MHz
        //input  wire        clk_sys_bak_50m_p     , // System clock, 50MHz

        //I2C interfaces
        output wire        fpga_i2c_en           , // FPGA I2C interface #1 enable, DIPSW ctrl
        inout  wire        fpga_i2c1_scl         , // FPGA I2C interface #1 SCL
        inout  wire        fpga_i2c1_sda         , // FPGA I2C interface #1 SDA
        output wire        fpga_i2c2_en          , // FPGA I2C interface #2 enable, DIPSW ctrl
        inout  wire        fpga_i2c2_scl         , // FPGA I2C interface #2 SCL
        inout  wire        fpga_i2c2_sda         , // FPGA I2C interface #2 SDA
        inout  wire        fpga_dimm_scl         , // FPGA I2C interface dedicated for RDIMM access
        inout  wire        fpga_dimm_sda         , // FPGA I2C interface dedicated for RDIMM access

        //GPIOs
        output wire [3:0]  fpga_led              , // FPGA user LEDs
      //input  wire        gxf_perstn            , // Dedicated reset from on-board PB, currently not used

        //XCVR reference clock
      //input  wire        refclk_fgt12ach0_p    , // QSFPDD reference clock 100MHz from U22
      //input  wire        refclk_fgt12ach3_p    , // QSFPDD reference clock 153.6MHz from U22
        input  wire        refclk_fgt12ach4_p    , // QSFPDD reference clock 156.25MHz from U22
      //input  wire        refclk_fgt12ach5_p    , // QSFPDD reference clock 156.25MHz from U22
      //input  wire        refclk_fgt12ach6_p    , // QSFPDD reference clock 184.32MHz from U22

        //QSFPDD LEDs
        output wire [2:0]  qsfpdd0_fpga_led      , // QSFPDD0 LEDs
        output wire [2:0]  qsfpdd1_fpga_led      , // QSFPDD1 LEDs

        //QSFPDD control signals
        output  wire       qsfpdd_12v_port_en    , // Enabling of I2C controller U14, external pull-up
        input   wire       qsfpdd_1v2_port_int_n , // Interrupt from I2C controller U14

        //QSFPDD XCVR channels
        input  wire [3:0]  qsfpdd0_rx_p          , // QSFPDD #0 RX P
        input  wire [3:0]  qsfpdd0_rx_n          , // QSFPDD #0 RX N
        output wire [3:0]  qsfpdd0_tx_p          , // QSFPDD #0 TX P
        output wire [3:0]  qsfpdd0_tx_n          ,  // QSFPDD #0 TX N
        input  wire [7:0]  qsfpdd1_rx_p          , // QSFPDD #1 RX P
        input  wire [7:0]  qsfpdd1_rx_n          , // QSFPDD #1 RX N
        output wire [7:0]  qsfpdd1_tx_p          , // QSFPDD #1 TX P
        output wire [7:0]  qsfpdd1_tx_n            // QSFPDD #1 TX N
        );

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Signal definitions
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
wire          ninit_done/* synthesis keep */;
wire          global_resetn/* synthesis keep */;
reg  [26:0]   heart_beat_cnt/* synthesis keep */;
wire          fpga_i2c1_scl_in/* synthesis keep */;
wire          fpga_i2c1_sda_in/* synthesis keep */;
wire          fpga_i2c1_scl_oe/* synthesis keep */;
wire          fpga_i2c1_sda_oe/* synthesis keep */;
wire          fpga_i2c2_scl_in/* synthesis keep */;
wire          fpga_i2c2_sda_in/* synthesis keep */;
wire          fpga_i2c2_scl_oe/* synthesis keep */;
wire          fpga_i2c2_sda_oe/* synthesis keep */;
wire          fpga_dimm_scl_in/* synthesis keep */;
wire          fpga_dimm_sda_in/* synthesis keep */;
wire          fpga_dimm_scl_oe/* synthesis keep */;
wire          fpga_dimm_sda_oe/* synthesis keep */;
wire          outclk0_100m     /* 100m           */;
wire          outclk1_50m      /* 50m            */;
wire          iopll_lock       /* synthesis keep */;
wire          global_resetn1   /* synthesis keep */;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Input/output assignments
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
assign fpga_i2c1_scl_in     = fpga_i2c1_scl;
assign fpga_i2c1_sda_in     = fpga_i2c1_sda;
assign fpga_i2c1_scl        = fpga_i2c1_scl_oe ? 1'b0 : 1'bz;
assign fpga_i2c1_sda        = fpga_i2c1_sda_oe ? 1'b0 : 1'bz;

assign fpga_i2c2_scl_in     = fpga_i2c2_scl;
assign fpga_i2c2_sda_in     = fpga_i2c2_sda;
assign fpga_i2c2_scl        = fpga_i2c2_scl_oe ? 1'b0 : 1'bz;
assign fpga_i2c2_sda        = fpga_i2c2_sda_oe ? 1'b0 : 1'bz;

assign fpga_dimm_scl_in     = fpga_dimm_scl;
assign fpga_dimm_sda_in     = fpga_dimm_sda;
assign fpga_dimm_scl        = fpga_dimm_scl_oe ? 1'b0 : 1'bz;
assign fpga_dimm_sda        = fpga_dimm_sda_oe ? 1'b0 : 1'bz;

assign fpga_i2c_en          = 1'bz; // DIPSW control externally
assign fpga_i2c2_en         = 1'bz; // DIPSW control externally

assign fpga_led[0]          = heart_beat_cnt[26];
assign fpga_led[1]          = heart_beat_cnt[26];
assign fpga_led[2]          = heart_beat_cnt[26];
assign fpga_led[3]          = heart_beat_cnt[26];

assign qsfpdd0_fpga_led     = 3'bzzz; // Only for debug, external pull-up
assign qsfpdd1_fpga_led     = 3'bzzz; // Only for debug, external pull-up
assign qsfpdd_12v_port_en   = 1'bz;   // Enabling of I2C controller U14, external pull-up

//----------------------------------------------------------------------------------------------------------
// Intel Agilex Reset Release
//----------------------------------------------------------------------------------------------------------
reset_release reset_release_inst 
              (
              .ninit_done (ninit_done)  // ninit_done.ninit_done
              );

assign global_resetn = !ninit_done & cpu_1v2_resetn;
assign global_resetn1 = global_resetn & iopll_lock;
//assign global_resetn = cpu_1v2_resetn;
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// iopll
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
iopll iopll_0 (
		.rst      (!global_resetn),      //   input,  width = 1,   reset.reset
		.refclk   (clk_sys_100m_p),   //   input,  width = 1,  refclk.clk
		.locked   (iopll_lock),   //  output,  width = 1,  locked.export
		.outclk_0 (outclk0_100m), //  output,  width = 1, outclk0.clk
		.outclk_1 (outclk1_50m)  //  output,  width = 1, outclk1.clk
	);
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Heart-beat indicate
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
always @(posedge outclk1_50m or negedge global_resetn1)
    if (!global_resetn1)
        heart_beat_cnt <= 27'h0; //0x7FFFFFF
    else
        heart_beat_cnt <= heart_beat_cnt + 1'b1;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//XCVR system Qsys module
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
q_sys   q_sys_u0 (
        .clk_100_clk                                                  (clk_sys_100m_p       ), //                                      clk_100.clk
        .reset_100_reset_n                                            (global_resetn1        ), //                                    reset_100.reset_n
        .clk_50_clk                                                   (outclk1_50m    ), //                                       clk_50.clk
        .reset_50_reset_n                                             (global_resetn1        ), //                                     reset_50.reset_n
        .i2c_0_i2c_serial_sda_in                                      (fpga_i2c1_sda_in     ), //                             i2c_0_i2c_serial.sda_in
        .i2c_0_i2c_serial_scl_in                                      (fpga_i2c1_scl_in     ), //                                             .scl_in
        .i2c_0_i2c_serial_sda_oe                                      (fpga_i2c1_sda_oe     ), //                                             .sda_oe
        .i2c_0_i2c_serial_scl_oe                                      (fpga_i2c1_scl_oe     ), //                                             .scl_oe
        .i2c_1_i2c_serial_sda_in                                      (fpga_i2c2_sda_in     ), //                             i2c_1_i2c_serial.sda_in
        .i2c_1_i2c_serial_scl_in                                      (fpga_i2c2_scl_in     ), //                                             .scl_in
        .i2c_1_i2c_serial_sda_oe                                      (fpga_i2c2_sda_oe     ), //                                             .sda_oe
        .i2c_1_i2c_serial_scl_oe                                      (fpga_i2c2_scl_oe     ), //                                             .scl_oe
        .i2c_2_i2c_serial_sda_in                                      (fpga_dimm_sda_in     ), //                             i2c_2_i2c_serial.sda_in
        .i2c_2_i2c_serial_scl_in                                      (fpga_dimm_scl_in     ), //                                             .scl_in
        .i2c_2_i2c_serial_sda_oe                                      (fpga_dimm_sda_oe     ), //                                             .sda_oe
        .i2c_2_i2c_serial_scl_oe                                      (fpga_dimm_scl_oe     ), //                                             .scl_oe
        .systemclk_f_0_refclk_fgt_in_refclk_fgt_4                     (refclk_fgt12ach4_p   ), //                     systemclk_f_0_refclk_fgt.in_refclk_fgt_0
      //.systemclk_f_1_refclk_fgt_in_refclk_fgt_5                     (refclk_fgt12ach5_p   ), //                     systemclk_f_1_refclk_fgt.in_refclk_fgt_0
        .ftile_qsfpdd0_xcvr_test_0_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd0_tx_p[0]      ), //     ftile_qsfpdd0_xcvr_test_0_tx_serial_data.tx_serial_data
        .ftile_qsfpdd0_xcvr_test_0_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd0_tx_n[0]      ), //   ftile_qsfpdd0_xcvr_test_0_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_0_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd0_rx_p[0]      ), //     ftile_qsfpdd0_xcvr_test_0_rx_serial_data.rx_serial_data
        .ftile_qsfpdd0_xcvr_test_0_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd0_rx_n[0]      ), //   ftile_qsfpdd0_xcvr_test_0_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_1_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd0_tx_p[1]      ), //     ftile_qsfpdd0_xcvr_test_1_tx_serial_data.tx_serial_data
        .ftile_qsfpdd0_xcvr_test_1_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd0_tx_n[1]      ), //   ftile_qsfpdd0_xcvr_test_1_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_1_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd0_rx_p[1]      ), //     ftile_qsfpdd0_xcvr_test_1_rx_serial_data.rx_serial_data
        .ftile_qsfpdd0_xcvr_test_1_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd0_rx_n[1]      ), //   ftile_qsfpdd0_xcvr_test_1_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_2_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd0_tx_p[2]      ), //     ftile_qsfpdd0_xcvr_test_2_tx_serial_data.tx_serial_data
        .ftile_qsfpdd0_xcvr_test_2_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd0_tx_n[2]      ), //   ftile_qsfpdd0_xcvr_test_2_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_2_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd0_rx_p[2]      ), //     ftile_qsfpdd0_xcvr_test_2_rx_serial_data.rx_serial_data
        .ftile_qsfpdd0_xcvr_test_2_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd0_rx_n[2]      ), //   ftile_qsfpdd0_xcvr_test_2_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_3_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd0_tx_p[3]      ), //     ftile_qsfpdd0_xcvr_test_3_tx_serial_data.tx_serial_data
        .ftile_qsfpdd0_xcvr_test_3_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd0_tx_n[3]      ), //   ftile_qsfpdd0_xcvr_test_3_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd0_xcvr_test_3_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd0_rx_p[3]      ), //     ftile_qsfpdd0_xcvr_test_3_rx_serial_data.rx_serial_data
        .ftile_qsfpdd0_xcvr_test_3_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd0_rx_n[3]      ), //   ftile_qsfpdd0_xcvr_test_3_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_0_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[0]      ), //     ftile_qsfpdd1_xcvr_test_0_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_0_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[0]      ), //   ftile_qsfpdd1_xcvr_test_0_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_0_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[0]      ), //     ftile_qsfpdd1_xcvr_test_0_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_0_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[0]      ), //   ftile_qsfpdd1_xcvr_test_0_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_1_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[1]      ), //     ftile_qsfpdd1_xcvr_test_1_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_1_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[1]      ), //   ftile_qsfpdd1_xcvr_test_1_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_1_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[1]      ), //     ftile_qsfpdd1_xcvr_test_1_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_1_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[1]      ), //   ftile_qsfpdd1_xcvr_test_1_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_2_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[2]      ), //     ftile_qsfpdd1_xcvr_test_2_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_2_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[2]      ), //   ftile_qsfpdd1_xcvr_test_2_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_2_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[2]      ), //     ftile_qsfpdd1_xcvr_test_2_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_2_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[2]      ), //   ftile_qsfpdd1_xcvr_test_2_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_3_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[3]      ), //     ftile_qsfpdd1_xcvr_test_3_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_3_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[3]      ), //   ftile_qsfpdd1_xcvr_test_3_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_3_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[3]      ), //     ftile_qsfpdd1_xcvr_test_3_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_3_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[3]      ), //   ftile_qsfpdd1_xcvr_test_3_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_4_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[4]      ), //     ftile_qsfpdd1_xcvr_test_0_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_4_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[4]      ), //   ftile_qsfpdd1_xcvr_test_0_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_4_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[4]      ), //     ftile_qsfpdd1_xcvr_test_0_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_4_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[4]      ), //   ftile_qsfpdd1_xcvr_test_0_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_5_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[5]      ), //     ftile_qsfpdd1_xcvr_test_1_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_5_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[5]      ), //   ftile_qsfpdd1_xcvr_test_1_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_5_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[5]      ), //     ftile_qsfpdd1_xcvr_test_1_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_5_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[5]      ), //   ftile_qsfpdd1_xcvr_test_1_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_6_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[6]      ), //     ftile_qsfpdd1_xcvr_test_2_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_6_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[6]      ), //   ftile_qsfpdd1_xcvr_test_2_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_6_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[6]      ), //     ftile_qsfpdd1_xcvr_test_2_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_6_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[6]      ), //   ftile_qsfpdd1_xcvr_test_2_rx_serial_data_n.rx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_7_directphy_f_0_tx_serial_data_tx_serial_data      (qsfpdd1_tx_p[7]      ), //     ftile_qsfpdd1_xcvr_test_3_tx_serial_data.tx_serial_data
        .ftile_qsfpdd1_xcvr_test_7_directphy_f_0_tx_serial_data_n_tx_serial_data_n  (qsfpdd1_tx_n[7]      ), //   ftile_qsfpdd1_xcvr_test_3_tx_serial_data_n.tx_serial_data_n
        .ftile_qsfpdd1_xcvr_test_7_directphy_f_0_rx_serial_data_rx_serial_data      (qsfpdd1_rx_p[7]      ), //     ftile_qsfpdd1_xcvr_test_3_rx_serial_data.rx_serial_data
        .ftile_qsfpdd1_xcvr_test_7_directphy_f_0_rx_serial_data_n_rx_serial_data_n  (qsfpdd1_rx_n[7]      ) //   ftile_qsfpdd1_xcvr_test_3_rx_serial_data_n.rx_serial_data_n
        
		);

endmodule
