/*****************************************************************************
 * INTEL CONFIDENTIAL
 * Copyright 2024 Intel Corporation.  All Rights Reserved.
 *
 * The source code contained or described herein and all documents related
 * to the source code ("Material") are owned by Intel Corporation or its
 * suppliers or licensors. Title to the Material remains with Intel
 * Corporation or its suppliers and licensors. The Material contains trade
 * secrets and proprietary and confidential information of Intel or its
 * suppliers and licensors. The Material is protected by worldwide copyright
 * and trade secret laws and treaty provisions. No part of the Material may
 * be used, copied, reproduced, modified, published, uploaded, posted,
 * transmitted, distributed, or disclosed in any way without Intel's prior
 * express written permission.
 *
 * No license under any patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or
 * delivery of the Materials, either expressly, by implication, inducement,
 * estoppel or otherwise. Any license under such intellectual property rights
 * must be express and approved by Intel in writing.
 *****************************************************************************/
 
#ifndef __FPC202_H__
#define __FPC202_H__

/* definition in FPC202 */
#define ITL_FPC202_REG_INTERVAL             0x20

#define ITL_FPC202_REG_IN_A                 0x06
#define ITL_FPC202_REG_IN_B                 0x07
#define ITL_FPC202_REG_OUT_AB_CONF          0x08
#define ITL_FPC202_REG_OUT_AB_VALUE         0x0A

#define ITL_FCP202_REG_LED_PWM              0x14
#define ITL_FPC202_REG_LED_BLINK_ON_TIME    0x16
#define ITL_FPC202_REG_LED_BLINK_OFF_TIME   0x17
#define ITL_FPC202_REG_LED_MODE             0x1A

#define ITL_FPC202_REG_GPIO_STAT            0x0F
#define ITL_FPC202_REG_GPIO01_CONF          0x96
#define ITL_FPC202_REG_GPIO23_CONF          0x97

//mask
#define ITL_FPC202_QSFP0_INTL_MASK          0x10
#define ITL_FPC202_QSFP1_INTL_MASK          0x40
#define ITL_FPC202_QSFP0_PRESENT_MASK       0x01
#define ITL_FPC202_QSFP1_PRESENT_MASK       0x04
#define ITL_FPC202_QSFP0_FAULT_MASK         0x02
#define ITL_FPC202_QSFP1_FAULT_MASK         0x08

//I2C 7-bit address
#define ITL_FPC202_ADDR_SELF (0x1E>>1)
#define ITL_FPC202_ADDR_QSFPDD0 (0xF0>>1)
#define ITL_FPC202_ADDR_QSFPDD1 (0xF8>>1)

#define ITL_FPC202_READ(addr,reg) \
    ret = itl_fpc202_out_read_reg(addr, reg, &data);\
    if(0 == ret)\
        printf("[%s] addr 0x%x reg %d= 0x%lx\n", #reg, addr, reg, data);\
    else        \
        printf("[%s] addr 0x%x reg %d failed![ERROR CODE:%lu]\n", #reg, addr, reg, ret);\

#define ITL_FPC202_WRITE(addr,reg,data) \
    ret = itl_fpc202_out_write_reg(addr, reg, data);\
    if(0 == ret)\
        printf("[%s] write addr 0x%x reg %d= 0x%lx success\n", #reg, addr, reg, data);\
    else        \
        printf("[%s] write addr 0x%x reg %d failed![ERROR CODE:%lu]\n", #reg, addr, reg, ret);\

typedef enum{
    P0_S0_A,
    P0_S0_B,
    P1_S0_A,
    P1_S0_B
}fpc202_port_enum_t;

void itl_fpc202_sync();
alt_u32 itl_fpc202_read_reg(alt_u16 reg, alt_u32 *data);
alt_u32 itl_fpc202_write_reg(alt_u16 reg, alt_u32 data);
alt_u32 itl_fpc202_out_read_reg(alt_u8 slave_addr, alt_u16 reg, alt_u32 *data);
alt_u32 itl_fpc202_out_write_reg(alt_u8 slave_addr, alt_u16 reg, alt_u32 data);

#endif
