/*****************************************************************************
 * INTEL CONFIDENTIAL
 * Copyright 2024 Intel Corporation.  All Rights Reserved.
 *
 * The source code contained or described herein and all documents related
 * to the source code ("Material") are owned by Intel Corporation or its
 * suppliers or licensors. Title to the Material remains with Intel
 * Corporation or its suppliers and licensors. The Material contains trade
 * secrets and proprietary and confidential information of Intel or its
 * suppliers and licensors. The Material is protected by worldwide copyright
 * and trade secret laws and treaty provisions. No part of the Material may
 * be used, copied, reproduced, modified, published, uploaded, posted,
 * transmitted, distributed, or disclosed in any way without Intel's prior
 * express written permission.
 *
 * No license under any patent, copyright, trade secret or other intellectual
 * property right is granted to or conferred upon you by disclosure or
 * delivery of the Materials, either expressly, by implication, inducement,
 * estoppel or otherwise. Any license under such intellectual property rights
 * must be express and approved by Intel in writing.
 *****************************************************************************/

#ifndef __I2C_INTERFACE_H__
#define __I2C_INTERFACE_H__

#include "altera_avalon_i2c.h"

#define ERR_PARAM    (-20)

#define ITL_I2C_0_BUS 0
#define ITL_I2C_1_BUS 1
#define ITL_I2C_2_BUS 2

#define ITL_TMP411_INLET_CORE_I2CADDR       0x4C
#define ITL_TMP411_EXAUST_XCVR_I2CADDR      0x4D
#define ITL_RDIMM0_I2CADDR                  0x18
#define ITL_RDIMM1_I2CADDR                  0x19
#define ITL_RDIMM2_I2CADDR                  0x1A
#define ITL_RDIMM3_I2CADDR                  0x1B
#define ITL_QSFP0_I2CADDR                   0x78
#define ITL_QSFP1_I2CADDR                   0x7C
#define ITL_EM2120_BMC_I2CADDR              0x43
#define ITL_EM2120_VCCERAM_I2CADDR          0x44
#define ITL_EM2120_VCCR_GXB_I2CADDR         0x45
#define ITL_EM2120_VCCT_GXB_I2CADDR         0x46
#define ITL_EM2120_FPGA_I2CADDR             0x47
#define ITL_MAX16545_PCIE_I2CADDR           0x42
#define ITL_MAX16545_AUX_I2CADDR            0x40
#define ITL_IDT5P49_DDR4CLK_I2CADDR         0x68
#define ITL_IDT5P49_FPGACLK_I2CADDR         0x6A
#define ITL_EEPROM_MAC_I2CADDR              0x57

#define ITL_TEMPERATURE_RESONABLE_HIGH      200
#define ITL_TEMPERATURE_RESONABLE_LOW       4

#define ITL_TMP411_CORE_STATUS_BIT          6
#define ITL_TMP411_XCVR_STATUS_BIT          5
#define ITL_RDIMM0_STATUS_BIT               16
#define ITL_RDIMM1_STATUS_BIT               17
#define ITL_RDIMM2_STATUS_BIT               18
#define ITL_RDIMM3_STATUS_BIT               19
#define ITL_QSFP0_STATUS_BIT                13
#define ITL_QSFP1_STATUS_BIT                14
#define ITL_ISL68137_STATUS_BIT             20
#define ITL_EM2120_BMC_STATUS_BIT           4
#define ITL_EM2120_VCCERAM_STATUS_BIT       3
#define ITL_EM2120_VCCR_STATUS_BIT          2
#define ITL_EM2120_VCCT_STATUS_BIT          1
#define ITL_EM2120_FPGA_STATUS_BIT          0
#define ITL_MAX16545_PCIE_STATUS_BIT        8
#define ITL_MAX16545_AUX_STATUS_BIT         7
#define ITL_IDT5P49_DDR4CLK_STATUS_BIT      10
#define ITL_IDT5P49_FPGACLK_STATUS_BIT      11
#define ITL_FPGA_I2C_SLAVE_STATUS_BIT       9
#define ITL_QSFP_CONTROLLER_STATUS_BIT      12
#define ITL_EEPROM_STATUS_BIT               15

#ifdef DEBUG
void dump_telemetry_data();
#endif

/*  @i2c_dev_id i2c master number (1, 2)
    @addr:          i2c slave address
    @reg:           register address, could be 8bit or 16bit width
    @*data:         the data read from i2c device
    @bitWidth:      accpt 8(bit), 16(bit), 24(bit) and 32(bit), other value would cause an error
    @return:        Status Code,   */
ALT_AVALON_I2C_STATUS_CODE itl_i2c_read_reg(int i2c_dev_id, alt_u8 addr, alt_u16 reg, alt_u32*data, alt_u8 bitwidth);

/*  @addr:          i2c slave address
    @reg:           register address, could be 8bit or 16bit width
    @data:      the data write to i2c device
    @bitWidth:      accpt 8(bit), 16(bit), 24(bit) and 32(bit), other value would cause an error
    @return:        Status Code,   */
ALT_AVALON_I2C_STATUS_CODE itl_i2c_write_reg(int i2c_dev_id, alt_u8 addr, alt_16 reg, alt_u32 data, alt_u8 bitwidth);

int itl_i2c_open_devices();

#endif
