
/*
 * "QSFPDD Module I2C Management" example.
 *
 * This example access QSFPDD modules by I2C multiplexer, and prints vendor
 * information to the STDOUT stream. It runs on the Nios V example designs.
 * It runs without the MicroC/OS-II RTOS and requires a STDOUT device in your
 * system's hardware.
 * The memory footprint of this hosted application is ~110 kbytes by default
 * using the standard reference design.
 *
 */

#include <stdio.h>
#include <string.h>
#include "unistd.h"
#include "priv/alt_busy_sleep.h"
#include "config.h"
#include "itl_i2c_interface.h"
#include "itl_fpc202.h"
#include "itl_qsfp.h"

static void _itl_get_qsfpdd_vendor_info(alt_u8 idx)
{
    alt_u32 data;
    alt_u8 vendorName[ITL_QSFP_VENDOR_NAME_LEN];
    alt_u8 vendorPN[ITL_QSFP_VENDOR_PN_LEN];
    alt_u8 vendorSN[ITL_QSFP_VENDOR_SN_LEN];
    const alt_u8 qsfpddModule[ITL_QSFPDD_NUM][8] = {"QSFPDD0", "QSFPDD1"};
    alt_u8 qsfpAddr = (idx == 0) ?ITL_FPC202_ADDR_QSFPDD0:ITL_FPC202_ADDR_QSFPDD1;

    memset(vendorName, 0, ITL_QSFP_VENDOR_NAME_LEN * sizeof(alt_u8));
    memset(vendorPN, 0, ITL_QSFP_VENDOR_PN_LEN * sizeof(alt_u8));
    memset(vendorSN, 0, ITL_QSFP_VENDOR_SN_LEN * sizeof(alt_u8));

    for(alt_u8 j = 0; j < ITL_QSFP_VENDOR_NAME_LEN; j++)
    {
        if(ITL_SUCCESS == itl_fpc202_out_read_reg(qsfpAddr, ITL_QSFP_REG_VENDOR_NAME + j, &data))
        {
            vendorName[j] = (data&0xff);
        }
    }

    for(alt_u8 j = 0; j < ITL_QSFP_VENDOR_PN_LEN; j++)
    {
        if(ITL_SUCCESS == itl_fpc202_out_read_reg(qsfpAddr, ITL_QSFP_REG_VENDOR_PN + j, &data))
        {
            vendorPN[j] = (data&0xff);
        }
    }

    for(alt_u8 j = 0; j < ITL_QSFP_VENDOR_SN_LEN; j++)
    {
        if(ITL_SUCCESS == itl_fpc202_out_read_reg(qsfpAddr, ITL_QSFP_REG_VENDOR_SN + j, &data))
        {
            vendorSN[j] = (data&0xff);
        }
    }

    printf("\n%s Vendor : %s    PN : %s    SN : %s\n", qsfpddModule[idx], vendorName, vendorPN, vendorSN);
}

int main()
{
    alt_u32 data = 0;
    ALT_AVALON_I2C_STATUS_CODE ret = 0;

    // Update firmware version
    printf("Agilex™ 7 FPGA I-Series Development Kit (2x R-Tile and 1x F-Tile) NIOSV firmware version: %X\n", ITL_BUILD_VERSION);
    ITL_IOWR_SYSINFO_FIRM_VERSION(ITL_SYS_INFO_BASE, ITL_BUILD_VERSION);
    ITL_IOWR_SYSINFO_QSFP_CONF_OUT(ITL_SYS_INFO_BASE, (ITL_QSFP_RESET_MASK | ITL_QSFP_RESET_MASK << ITL_QSFP1_SHIFT));
    if(itl_i2c_open_devices() < 0)
    {
        return ITL_FAIL;
    }

    itl_fpc202_sync();
    usleep(1000*200);

    ITL_FPC202_READ(ITL_FPC202_ADDR_SELF, ITL_FPC202_REG_IN_A);
    ITL_FPC202_READ(ITL_FPC202_ADDR_SELF, ITL_FPC202_REG_IN_B);
    ITL_FPC202_READ(ITL_FPC202_ADDR_SELF, ITL_FPC202_REG_OUT_AB_CONF);
    ITL_FPC202_READ(ITL_FPC202_ADDR_SELF, ITL_FPC202_REG_OUT_AB_VALUE);

    ITL_FPC202_WRITE(ITL_FPC202_ADDR_SELF, ITL_FPC202_REG_OUT_AB_CONF, (alt_u32)0x55);
    usleep(1000*1000);

    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD0, ITL_QSFP_REG_MANAGECHARACTER0);
    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD0, ITL_QSFP_REG_MANAGECHARACTER1);
    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD0, ITL_QSFP_REG_MANAGECHARACTER2);
    _itl_get_qsfpdd_vendor_info(0);

    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD1, ITL_QSFP_REG_MANAGECHARACTER0);
    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD1, ITL_QSFP_REG_MANAGECHARACTER1);
    ITL_FPC202_READ(ITL_FPC202_ADDR_QSFPDD1, ITL_QSFP_REG_MANAGECHARACTER2);
    _itl_get_qsfpdd_vendor_info(1);

    while(1)
    {
        usleep(1);
    }
    return ITL_SUCCESS;
}

